@@ -21,12 +21,13 @@ android {
             jniLibs.srcDirs("libs")
         }
     }
-    packagingOptions {
+    packaging {
         resources.excludes += "META-INF/robovm/ios/robovm.xml"
         // part of kotlinx-coroutines-android, should not go into the apk
         resources.excludes += "DebugProbesKt.bin"
     }
     defaultConfig {
+        namespace = "com.unciv.app"
         applicationId = "com.unciv.app"
         minSdk = 21
         targetSdk = 33 // See #5044
@@ -46,7 +47,7 @@ android {
 
     signingConfigs {
         getByName("debug") {
-            storeFile = rootProject.file("debug.keystore")
+            storeFile = rootProject.file("android/debug.keystore")
             keyAlias = "androiddebugkey"
             keyPassword = "android"
             storePassword = "android"
@@ -54,27 +55,32 @@ android {
     }
 
     buildTypes {
-        getByName("release") {
+        debug {
+            isDebuggable = true
+        }
+        release {
             // If you make this true you get a version of the game that just flat-out doesn't run
             isMinifyEnabled = false
             proguardFiles(getDefaultProguardFile("proguard-android.txt"), "proguard-rules.pro")
+            isDebuggable = false
         }
-
     }
+
     lint {
         disable += "MissingTranslation"   // see res/values/strings.xml
     }
     compileOptions {
-        sourceCompatibility = JavaVersion.VERSION_1_8
         targetCompatibility = JavaVersion.VERSION_1_8
-
         isCoreLibraryDesugaringEnabled = true
     }
     androidResources {
         // Don't add local save files and fonts to release, obviously
         ignoreAssetsPattern = "!SaveFiles:!fonts:!maps:!music:!mods"
     }
-    buildToolsVersion = "33.0.2"
+    buildFeatures {
+        renderScript = true
+        aidl = true
+    }
 }
 
 task("texturePacker") {
@@ -135,11 +141,8 @@ tasks.register<JavaExec>("run") {
 }
 
 dependencies {
-    // Updating to latest version would require upgrading sourceCompatibility and targetCompatibility to 1_8, and targetSdk to 31 -
-    //   run `./gradlew build --scan` to see details
-    // Known Android Lint warning: "GradleDependency"
-    implementation("androidx.core:core-ktx:1.7.0")
-    implementation("androidx.work:work-runtime-ktx:2.7.1")
+    implementation("androidx.core:core-ktx:1.10.1")
+    implementation("androidx.work:work-runtime-ktx:2.8.1")
     // Needed to convert e.g. Android 26 API calls to Android 21
     // If you remove this run `./gradlew :android:lintDebug` to ensure everything's okay.
     // If you want to upgrade this, check it's working by building an apk,
