@@ -30,16 +30,15 @@ import com.unciv.logic.GameInfo
 import com.unciv.logic.files.UncivFiles
 import com.unciv.logic.multiplayer.storage.FileStorageRateLimitReached
 import com.unciv.logic.multiplayer.storage.OnlineMultiplayerServer
-import com.unciv.models.metadata.GameSettingsMultiplayer
-import kotlinx.coroutines.runBlocking
+import com.unciv.models.metadata.GameSettings.GameSettingsMultiplayer
 import java.io.FileNotFoundException
 import java.io.PrintWriter
 import java.io.StringWriter
 import java.io.Writer
 import java.time.Duration
-import java.util.Arrays
 import java.util.GregorianCalendar
 import java.util.concurrent.TimeUnit
+import kotlinx.coroutines.runBlocking
 
 
 class MultiplayerTurnCheckWorker(appContext: Context, workerParams: WorkerParameters)
@@ -69,13 +68,12 @@ class MultiplayerTurnCheckWorker(appContext: Context, workerParams: WorkerParame
         private const val FILE_STORAGE = "FILE_STORAGE"
         private const val AUTH_HEADER = "AUTH_HEADER"
 
+        private val constraints = Constraints.Builder()
+            // If no internet is available, worker waits before becoming active.
+            .setRequiredNetworkType(NetworkType.CONNECTED)
+            .build()
+
         fun enqueue(appContext: Context, delay: Duration, inputData: Data) {
-
-            val constraints = Constraints.Builder()
-                    // If no internet is available, worker waits before becoming active.
-                    .setRequiredNetworkType(NetworkType.CONNECTED)
-                    .build()
-
             val checkTurnWork = OneTimeWorkRequestBuilder<MultiplayerTurnCheckWorker>()
                     .setConstraints(constraints)
                     .setInitialDelay(delay.seconds, TimeUnit.SECONDS)
@@ -191,7 +189,11 @@ class MultiplayerTurnCheckWorker(appContext: Context, workerParams: WorkerParame
 
         fun startTurnChecker(applicationContext: Context, files: UncivFiles, currentGameInfo: GameInfo, settings: GameSettingsMultiplayer) {
             Log.i(LOG_TAG, "startTurnChecker")
+
+            // Games that haven't been updated in a week are considered stale
+            val oneWeekWorthOfMilliseconds = 1000*60*60*24*7
             val gameFiles = files.getMultiplayerSaves()
+                .filter { it.lastModified() > System.currentTimeMillis() - oneWeekWorthOfMilliseconds }
             val gameIds = Array(gameFiles.count()) {""}
             val gameNames = Array(gameFiles.count()) {""}
 
@@ -208,6 +210,7 @@ class MultiplayerTurnCheckWorker(appContext: Context, workerParams: WorkerParame
                     //just skip one file
                 }
             }
+            if (count==0) return // no games to update
 
             Log.d(LOG_TAG, "start gameNames: ${gameNames.contentToString()}")
 
@@ -294,7 +297,7 @@ class MultiplayerTurnCheckWorker(appContext: Context, workerParams: WorkerParame
             // to download all games to update the files so we save the first one we find
             var foundGame: Pair<String, String>? = null
 
-            for (idx in gameIds.indices){
+            for (idx in gameIds.indices) {
                 val gameId = gameIds[idx]
                 //gameId could be an empty string if startTurnChecker fails to load all files
                 if (gameId.isEmpty())
@@ -330,7 +333,7 @@ class MultiplayerTurnCheckWorker(appContext: Context, workerParams: WorkerParame
                     Log.i(LOG_TAG, "doWork FileStorageRateLimitReached ${ex.message}")
                     // We just break here as the configured delay is probably enough to wait for the rate limit anyway
                     break
-                } catch (ex: FileNotFoundException){
+                } catch (ex: FileNotFoundException) {
                     Log.i(LOG_TAG, "doWork FileNotFoundException ${ex.message}")
                     // FileNotFoundException is thrown by OnlineMultiplayer().tryDownloadGamePreview(gameId)
                     // and indicates that there is no game preview present for this game
@@ -339,7 +342,7 @@ class MultiplayerTurnCheckWorker(appContext: Context, workerParams: WorkerParame
                 }
             }
 
-            if (foundGame != null){
+            if (foundGame != null) {
                 notifyUserAboutTurn(applicationContext, foundGame)
                 with(NotificationManagerCompat.from(applicationContext)) {
                     cancel(NOTIFICATION_ID_SERVICE)
@@ -369,7 +372,7 @@ class MultiplayerTurnCheckWorker(appContext: Context, workerParams: WorkerParame
                 val inputDataFailIncrease = Data.Builder().putAll(inputData).putInt(FAIL_COUNT, failCount + 1).build()
                 enqueue(applicationContext, Duration.ofMinutes(1), inputDataFailIncrease)
             }
-        } catch (outOfMemory: OutOfMemoryError){ // no point in trying multiple times if this was an oom error
+        } catch (outOfMemory: OutOfMemoryError) { // no point in trying multiple times if this was an oom error
             Log.e(LOG_TAG, "doWork ${outOfMemory::class.simpleName}: ${outOfMemory.message}")
             return@runBlocking Result.failure()
         }
