@@ -171,8 +171,7 @@ object BackwardCompatibility {
     fun GameInfo.guaranteeUnitPromotions() {
         for (tileInfo in tileMap.values) for (unit in tileInfo.getUnits())
             for (startingPromo in unit.baseUnit.promotions)
-                if (startingPromo !in unit.promotions.promotions)
-                    unit.promotions.addPromotion(startingPromo, true)
+                unit.promotions.addPromotion(startingPromo, true)
     }
 
     /** Move max XP from barbarians to new home */
@@ -201,11 +200,11 @@ object BackwardCompatibility {
         historyStartTurn = turns
     }
 
-    fun GameInfo.ensureUnitIds(){
+    fun GameInfo.ensureUnitIds() {
         if (lastUnitId == 0) lastUnitId = tileMap.values.asSequence()
-            .flatMap { it.getUnits() }.maxOfOrNull { it.id } ?: 0
+            .flatMap { it.getUnits() }.maxOfOrNull { it.id }?.coerceAtLeast(0) ?: 0
         for (unit in tileMap.values.flatMap { it.getUnits() }) {
-            if (unit.id == 0) unit.id = ++lastUnitId
+            if (unit.id == Constants.NO_ID) unit.id = ++lastUnitId
         }
     }
 }
