@@ -33,7 +33,7 @@ object NextTurnAutomation {
 
     /** Top-level AI turn task list */
     fun automateCivMoves(civInfo: Civilization) {
-        if (civInfo.isBarbarian()) return BarbarianAutomation(civInfo).automate()
+        if (civInfo.isBarbarian) return BarbarianAutomation(civInfo).automate()
         if (civInfo.isSpectator()) return // When there's a spectator in multiplayer games, it's processed automatically, but shouldn't be able to actually do anything
 
         respondToPopupAlerts(civInfo)
@@ -64,7 +64,7 @@ object NextTurnAutomation {
         chooseTechToResearch(civInfo)
         automateCityBombardment(civInfo)
         UseGoldAutomation.useGold(civInfo)
-        if (!civInfo.isCityState()) {
+        if (!civInfo.isCityState) {
             protectCityStates(civInfo)
             bullyCityStates(civInfo)
         }
@@ -104,14 +104,14 @@ object NextTurnAutomation {
         for (popupAlert in civInfo.popupAlerts.toList()) { // toList because this can trigger other things that give alerts, like Golden Age
             if (popupAlert.type == AlertType.DemandToStopSettlingCitiesNear) {  // we're called upon to make a decision
                 val demandingCiv = civInfo.gameInfo.getCivilization(popupAlert.value)
-                val diploManager = civInfo.getDiplomacyManager(demandingCiv)
+                val diploManager = civInfo.getDiplomacyManager(demandingCiv)!!
                 if (Automation.threatAssessment(civInfo, demandingCiv) >= ThreatLevel.High)
                     diploManager.agreeNotToSettleNear()
                 else diploManager.refuseDemandNotToSettleNear()
             }
             if (popupAlert.type == AlertType.DeclarationOfFriendship) {
                 val requestingCiv = civInfo.gameInfo.getCivilization(popupAlert.value)
-                val diploManager = civInfo.getDiplomacyManager(requestingCiv)
+                val diploManager = civInfo.getDiplomacyManager(requestingCiv)!!
                 if (civInfo.diplomacyFunctions.canSignDeclarationOfFriendshipWith(requestingCiv)
                     && DiplomacyAutomation.wantsToSignDeclarationOfFrienship(civInfo,requestingCiv)) {
                     diploManager.signDeclarationOfFriendship()
@@ -179,7 +179,7 @@ object NextTurnAutomation {
 
         // The more we have invested into the city-state the more the alliance is worth
         val ourInfluence = if (civInfo.knows(cityState))
-            cityState.getDiplomacyManager(civInfo).getInfluence().toInt()
+            cityState.getDiplomacyManager(civInfo)!!.getInfluence().toInt()
         else 0
         value += ourInfluence / 10
 
@@ -190,7 +190,7 @@ object NextTurnAutomation {
 
         if (cityState.getAllyCiv() != null && cityState.getAllyCiv() != civInfo.civName) {
             // easier not to compete if a third civ has this locked down
-            val thirdCivInfluence = cityState.getDiplomacyManager(cityState.getAllyCiv()!!).getInfluence().toInt()
+            val thirdCivInfluence = cityState.getDiplomacyManager(cityState.getAllyCiv()!!)!!.getInfluence().toInt()
             value -= (thirdCivInfluence - 30) / 10
         }
 
@@ -209,8 +209,8 @@ object NextTurnAutomation {
     }
 
     private fun protectCityStates(civInfo: Civilization) {
-        for (state in civInfo.getKnownCivs().filter { !it.isDefeated() && it.isCityState() }) {
-            val diplomacyManager = state.getDiplomacyManager(civInfo.civName)
+        for (state in civInfo.getKnownCivs().filter { !it.isDefeated() && it.isCityState }) {
+            val diplomacyManager = state.getDiplomacyManager(civInfo.civName)!!
             val isAtLeastFriend = diplomacyManager.isRelationshipLevelGE(RelationshipLevel.Friend)
             if (isAtLeastFriend && state.cityStateFunctions.otherCivCanPledgeProtection(civInfo)) {
                 state.cityStateFunctions.addProtectorCiv(civInfo)
@@ -221,8 +221,8 @@ object NextTurnAutomation {
     }
 
     private fun bullyCityStates(civInfo: Civilization) {
-        for (state in civInfo.getKnownCivs().filter { !it.isDefeated() && it.isCityState() }.toList()) {
-            val diplomacyManager = state.getDiplomacyManager(civInfo.civName)
+        for (state in civInfo.getKnownCivs().filter { !it.isDefeated() && it.isCityState }.toList()) {
+            val diplomacyManager = state.getDiplomacyManager(civInfo.civName)!!
             if (diplomacyManager.isRelationshipLevelLT(RelationshipLevel.Friend)
                     && diplomacyManager.diplomaticStatus == DiplomaticStatus.Peace
                     && valueCityStateAlliance(civInfo, state) <= 0
@@ -434,7 +434,7 @@ object NextTurnAutomation {
         val ownMilitaryStrength = civInfo.getStatForRanking(RankingType.Force)
         val sumOfEnemiesMilitaryStrength =
                 civInfo.gameInfo.civilizations
-                    .filter { it != civInfo && !it.isBarbarian() && civInfo.isAtWarWith(it) }
+                    .filter { it != civInfo && !it.isBarbarian && civInfo.isAtWarWith(it) }
                     .sumOf { it.getStatForRanking(RankingType.Force) }
         val civHasSignificantlyWeakerMilitaryThanEnemies =
                 ownMilitaryStrength < sumOfEnemiesMilitaryStrength * 0.66f
@@ -459,7 +459,8 @@ object NextTurnAutomation {
 
     private fun trainSettler(civInfo: Civilization) {
         val personality = civInfo.getPersonality()
-        if (civInfo.isCityState()) return
+        if (civInfo.isCityState) return
+        if (civInfo.isOneCityChallenger()) return
         if (civInfo.isAtWar()) return // don't train settlers when you could be training troops.
         if (civInfo.wantsToFocusOn(Victory.Focus.Culture) && civInfo.cities.size > 3 &&
             civInfo.getPersonality().isNeutralPersonality)
@@ -467,19 +468,18 @@ object NextTurnAutomation {
         if (civInfo.cities.none()) return
         if (civInfo.getHappiness() <= civInfo.cities.size) return
 
+        if (civInfo.units.getCivUnits().any { it.hasUnique(UniqueType.FoundCity) }) return
+        if (civInfo.cities.any {
+                val currentConstruction = it.cityConstructions.getCurrentConstruction()
+                currentConstruction is BaseUnit && currentConstruction.isCityFounder()
+            }) return
         val settlerUnits = civInfo.gameInfo.ruleset.units.values
                 .filter { it.isCityFounder() && it.isBuildable(civInfo) &&
                     personality.getMatchingUniques(UniqueType.WillNotBuild, StateForConditionals(civInfo))
                         .none { unique -> it.matchesFilter(unique.params[0]) } }
         if (settlerUnits.isEmpty()) return
-        if (!civInfo.units.getCivUnits().none { it.hasUnique(UniqueType.FoundCity) }) return
 
-        if (civInfo.cities.any {
-                val currentConstruction = it.cityConstructions.getCurrentConstruction()
-                currentConstruction is BaseUnit && currentConstruction.isCityFounder()
-            }) return
-
-        if (civInfo.units.getCivUnits().none { it.isMilitary() }) return // We need someone to defend him first
+        if (civInfo.units.getCivUnits().count { it.isMilitary() } < civInfo.cities.size) return // We need someone to defend them first
 
         val workersBuildableForThisCiv = civInfo.gameInfo.ruleset.units.values.any {
             it.hasUnique(UniqueType.BuildImprovements)
@@ -509,14 +509,14 @@ object NextTurnAutomation {
             val knownMajorCivs = civ.getKnownCivs().filter { it.isMajorCiv() }
             val highestOpinion = knownMajorCivs
                 .maxOfOrNull {
-                    civ.getDiplomacyManager(it).opinionOfOtherCiv()
+                    civ.getDiplomacyManager(it)!!.opinionOfOtherCiv()
                 }
 
             if (highestOpinion == null) null  // Abstain if we know nobody
             else if (highestOpinion < -80 || highestOpinion < -40 && highestOpinion + Random.Default.nextInt(40) < -40)
                 null // Abstain if we hate everybody (proportional chance in the RelationshipLevel.Enemy range - lesser evil)
             else knownMajorCivs
-                .filter { civ.getDiplomacyManager(it).opinionOfOtherCiv() == highestOpinion }
+                .filter { civ.getDiplomacyManager(it)!!.opinionOfOtherCiv() == highestOpinion }
                 .toList().random().civName
 
         } else {
@@ -528,14 +528,14 @@ object NextTurnAutomation {
 
     private fun issueRequests(civInfo: Civilization) {
         for (otherCiv in civInfo.getKnownCivs().filter { it.isMajorCiv() && !civInfo.isAtWarWith(it) }) {
-            val diploManager = civInfo.getDiplomacyManager(otherCiv)
+            val diploManager = civInfo.getDiplomacyManager(otherCiv)!!
             if (diploManager.hasFlag(DiplomacyFlags.SettledCitiesNearUs))
                 onCitySettledNearBorders(civInfo, otherCiv)
         }
     }
 
     private fun onCitySettledNearBorders(civInfo: Civilization, otherCiv: Civilization) {
-        val diplomacyManager = civInfo.getDiplomacyManager(otherCiv)
+        val diplomacyManager = civInfo.getDiplomacyManager(otherCiv)!!
         when {
             diplomacyManager.hasFlag(DiplomacyFlags.IgnoreThemSettlingNearUs) -> {
             }
