@@ -145,7 +145,8 @@ class CityStats(val city: City) {
     private fun addStatPercentBonusesFromBuildings(statPercentBonusTree: StatTreeNode) {
         val localUniqueCache = LocalUniqueCache()
         for (building in city.cityConstructions.getBuiltBuildings())
-            statPercentBonusTree.addStats(building.getStatPercentageBonuses(city, localUniqueCache), "Buildings", building.name)
+            statPercentBonusTree.addStats(building.getStatPercentageBonuses(city, localUniqueCache),
+                "Buildings", building.name)
     }
 
     private fun getStatPercentBonusesFromPuppetCity(): Stats {
@@ -266,7 +267,7 @@ class CityStats(val city: City) {
                     city.getMatchingUniques(UniqueType.PercentProductionWonders)
                 currentConstruction is Building && !currentConstruction.isAnyWonder() ->
                     city.getMatchingUniques(UniqueType.PercentProductionBuildings)
-                else -> sequenceOf() // Science/Gold production
+                else -> emptySequence() // Science/Gold production
             }
 
         for (unique in uniquesToCheck) {
@@ -637,14 +638,4 @@ class CityStats(val city: City) {
     }
 
     //endregion
-
-    fun getStatDifferenceFromBuilding(building: String): Stats {
-        val newCity = city.clone()
-        newCity.setTransients(city.civ) // Will break the owned tiles. Needs to be reverted before leaving this function
-        newCity.cityConstructions.builtBuildings.add(building)
-        newCity.cityConstructions.setTransients()
-        newCity.cityStats.update(updateCivStats = false)
-        city.expansion.setTransients() // Revert owned tiles to original city
-        return newCity.cityStats.currentCityStats - currentCityStats
-    }
 }
