@@ -1,7 +1,6 @@
 package com.unciv.logic.city.managers
 
 import com.badlogic.gdx.math.Vector2
-import com.unciv.Constants
 import com.unciv.logic.IsPartOfGameInfoSerialization
 import com.unciv.logic.automation.Automation
 import com.unciv.logic.city.City
@@ -14,7 +13,6 @@ import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.ui.components.extensions.toPercent
 import com.unciv.ui.components.extensions.withItem
 import com.unciv.ui.components.extensions.withoutItem
-import com.unciv.ui.screens.worldscreen.unit.actions.UnitActions
 import kotlin.math.max
 import kotlin.math.pow
 import kotlin.math.roundToInt
@@ -47,7 +45,7 @@ class CityExpansionManager : IsPartOfGameInfoSerialization {
 
         cultureToNextTile *= city.civ.gameInfo.speed.cultureCostModifier
 
-        if (city.civ.isCityState())
+        if (city.civ.isCityState)
             cultureToNextTile *= 1.5f   // City states grow slower, perhaps 150% cost?
 
         for (unique in city.getMatchingUniques(UniqueType.BorderGrowthPercentage))
@@ -191,7 +189,7 @@ class CityExpansionManager : IsPartOfGameInfoSerialization {
                 // So lets try to wake it up for the player to notice it
                 if (unit.cache.hasUniqueToBuildImprovements || unit.cache.hasUniqueToCreateWaterImprovements) {
                     unit.due = true
-                    unit.action = null;
+                    unit.action = null
                 }
             }
 
