@@ -1,18 +1,20 @@
 package com.unciv.logic.civilization.managers
 
-import com.unciv.Constants
 import com.unciv.logic.IsPartOfGameInfoSerialization
 import com.unciv.logic.city.City
 import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.map.mapunit.MapUnit
+import com.unciv.logic.map.tile.Tile
 import com.unciv.models.Counter
 import com.unciv.models.Religion
 import com.unciv.models.ruleset.Belief
 import com.unciv.models.ruleset.BeliefType
+import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.UniqueTriggerActivation
 import com.unciv.models.ruleset.unique.UniqueType
+import com.unciv.models.ruleset.unit.BaseUnit
 import com.unciv.ui.components.extensions.toPercent
-import java.lang.Integer.max
+import com.unciv.ui.screens.worldscreen.unit.actions.UnitActionModifiers
 import java.lang.Integer.min
 import kotlin.random.Random
 
@@ -134,16 +136,15 @@ class ReligionManager : IsPartOfGameInfoSerialization {
         civInfo.gameInfo.religions[beliefName] = religion!!
         for (city in civInfo.cities)
             city.religion.addPressure(beliefName, 200 * city.population.population)
-        religionState = ReligionState.Pantheon
-
-        for (unique in civInfo.getTriggeredUniques(UniqueType.TriggerUponFoundingPantheon))
-            UniqueTriggerActivation.triggerCivwideUnique(unique, civInfo)
     }
 
+    fun greatProphetsEarned(): Int = civInfo.civConstructions.boughtItemsWithIncreasingPrice[getGreatProphetEquivalent()?.name ?: ""]
+        // Counter.get never returns null, but it needs the correct key type, which is non-nullable
+
     // https://www.reddit.com/r/civ/comments/2m82wu/can_anyone_detail_the_finer_points_of_great/
     // Game files (globaldefines.xml)
     fun faithForNextGreatProphet(): Int {
-        val greatProphetsEarned = civInfo.civConstructions.boughtItemsWithIncreasingPrice[getGreatProphetEquivalent()!!]
+        val greatProphetsEarned = greatProphetsEarned()
 
         var faithCost =
             (200 + 100 * greatProphetsEarned * (greatProphetsEarned + 1) / 2f) *
@@ -166,12 +167,13 @@ class ReligionManager : IsPartOfGameInfoSerialization {
         return true
     }
 
-    fun getGreatProphetEquivalent(): String? {
-        return civInfo.gameInfo.ruleset.units.values.firstOrNull { it.hasUnique(UniqueType.MayFoundReligion) }?.name
+    fun getGreatProphetEquivalent(): BaseUnit? {
+        val baseUnit = civInfo.gameInfo.ruleset.units.values.firstOrNull { it.hasUnique(UniqueType.MayFoundReligion) }
+        return if (baseUnit == null) null else civInfo.getEquivalentUnit(baseUnit)
     }
 
     private fun generateProphet() {
-        val prophetUnitName = getGreatProphetEquivalent() ?: return // No prophet units in this mod
+        val prophetUnit = getGreatProphetEquivalent() ?: return // No prophet units in this mod
 
         val prophetSpawnChange = (5f + storedFaith - faithForNextGreatProphet()) / 100f
 
@@ -179,10 +181,10 @@ class ReligionManager : IsPartOfGameInfoSerialization {
             val birthCity =
                 if (religionState <= ReligionState.Pantheon) civInfo.getCapital()
                 else civInfo.religionManager.getHolyCity()
-            val prophet = civInfo.units.addUnit(prophetUnitName, birthCity) ?: return
+            val prophet = civInfo.units.addUnit(prophetUnit, birthCity) ?: return
             prophet.religion = religion!!.name
             storedFaith -= faithForNextGreatProphet()
-            civInfo.civConstructions.boughtItemsWithIncreasingPrice.add(prophetUnitName, 1)
+            civInfo.civConstructions.boughtItemsWithIncreasingPrice.add(prophetUnit.name, 1)
         }
     }
 
@@ -254,11 +256,9 @@ class ReligionManager : IsPartOfGameInfoSerialization {
     }
 
 
-    fun mayFoundReligionAtAll(prophet: MapUnit): Boolean {
+    fun mayFoundReligionAtAll(): Boolean {
         if (!civInfo.gameInfo.isReligionEnabled()) return false // No religion
         if (religionState >= ReligionState.Religion) return false // Already created a major religion
-        // Already used its power for other things
-        if (prophet.abilityUsesLeft.any { it.value != prophet.maxAbilityUses[it.key] }) return false
 
         if (!civInfo.isMajorCiv()) return false // Only major civs may use religion
 
@@ -268,30 +268,25 @@ class ReligionManager : IsPartOfGameInfoSerialization {
         return true
     }
 
-    fun mayFoundReligionNow(prophet: MapUnit): Boolean {
-        if (!mayFoundReligionAtAll(prophet)) return false
-        if (!prophet.getTile().isCityCenter()) return false
-        if (prophet.getTile().getCity()!!.isHolyCity()) return false
+    fun mayFoundReligionHere(tile: Tile): Boolean {
+        if (!mayFoundReligionAtAll()) return false
+        if (!tile.isCityCenter()) return false
+        if (tile.getCity()!!.isHolyCity()) return false
         // No double holy cities. Not sure if these were allowed in the base game
         return true
     }
 
     fun foundReligion(prophet: MapUnit) {
-        if (!mayFoundReligionNow(prophet)) return // How did you do this?
+        if (!mayFoundReligionHere(prophet.getTile())) return // How did you do this?
         if (religionState == ReligionState.None) shouldChoosePantheonBelief = true
         religionState = ReligionState.FoundingReligion
         civInfo.religionManager.foundingCityId = prophet.getTile().getCity()!!.id
-
-        for (unique in civInfo.getTriggeredUniques(UniqueType.TriggerUponFoundingReligion))
-            UniqueTriggerActivation.triggerCivwideUnique(unique, civInfo)
     }
 
-    fun mayEnhanceReligionAtAll(prophet: MapUnit): Boolean {
-        if (!civInfo.gameInfo.isReligionEnabled()) return false // No religion, no enhancing
+    fun mayEnhanceReligionAtAll(): Boolean {
+        if (!civInfo.gameInfo.isReligionEnabled()) return false
         if (religion == null) return false // First found a pantheon
         if (religionState != ReligionState.Religion) return false // First found an actual religion
-        // Already used its power for other things
-        if (prophet.abilityUsesLeft.any { it.value != prophet.maxAbilityUses[it.key] }) return false
         if (!civInfo.isMajorCiv()) return false // Only major civs
 
         if (numberOfBeliefsAvailable(BeliefType.Follower) == 0)
@@ -303,18 +298,15 @@ class ReligionManager : IsPartOfGameInfoSerialization {
         return true
     }
 
-    fun mayEnhanceReligionNow(prophet: MapUnit): Boolean {
-        if (!mayEnhanceReligionAtAll(prophet)) return false
-        if (!prophet.getTile().isCityCenter()) return false
+    fun mayEnhanceReligionHere(tile: Tile): Boolean {
+        if (!mayEnhanceReligionAtAll()) return false
+        if (!tile.isCityCenter()) return false
         return true
     }
 
     fun useProphetForEnhancingReligion(prophet: MapUnit) {
-        if (!mayEnhanceReligionNow(prophet)) return // How did you do this?
+        if (!mayEnhanceReligionHere(prophet.getTile())) return // How did you do this?
         religionState = ReligionState.EnhancingReligion
-
-        for (unique in civInfo.getTriggeredUniques(UniqueType.TriggerUponEnhancingReligion))
-            UniqueTriggerActivation.triggerCivwideUnique(unique, civInfo)
     }
 
     /**
@@ -380,36 +372,39 @@ class ReligionManager : IsPartOfGameInfoSerialization {
         // Must be done first in case when gain more later
         freeBeliefs.clear()
 
+        if (religionState == ReligionState.None)
+            foundPantheon(beliefs[0].name, useFreeBeliefs)  // makes religion non-null
+        // add beliefs (religion exists at this point)
+        religion!!.addBeliefs(beliefs)
+
         when (religionState) {
+            ReligionState.None -> {
+                religionState = ReligionState.Pantheon
+                for (unique in civInfo.getTriggeredUniques(UniqueType.TriggerUponFoundingPantheon))
+                    UniqueTriggerActivation.triggerUnique(unique, civInfo)
+            }
+            ReligionState.FoundingReligion -> {
+                religionState = ReligionState.Religion
+                for (unique in civInfo.getTriggeredUniques(UniqueType.TriggerUponFoundingReligion))
+                    UniqueTriggerActivation.triggerUnique(unique, civInfo)
+            }
             ReligionState.EnhancingReligion -> {
                 religionState = ReligionState.EnhancedReligion
-            }
-            ReligionState.None -> {
-                foundPantheon(beliefs[0].name, useFreeBeliefs)
+                for (unique in civInfo.getTriggeredUniques(UniqueType.TriggerUponEnhancingReligion))
+                    UniqueTriggerActivation.triggerUnique(unique, civInfo)
             }
             else -> {}
         }
-        // add beliefs (religion exists at this point)
-        religion!!.followerBeliefs.addAll(
-            beliefs
-                .filter { it.type == BeliefType.Pantheon || it.type == BeliefType.Follower }
-                .map { it.name }
-        )
-        religion!!.founderBeliefs.addAll(
-            beliefs
-                .filter { it.type == BeliefType.Founder || it.type == BeliefType.Enhancer }
-                .map { it.name }
-        )
 
         for (unique in civInfo.getTriggeredUniques(UniqueType.TriggerUponAdoptingPolicyOrBelief))
             for (belief in beliefs)
                 if (unique.conditionals.any {it.type == UniqueType.TriggerUponAdoptingPolicyOrBelief && it.params[0] == belief.name})
-                    UniqueTriggerActivation.triggerCivwideUnique(unique, civInfo,
+                    UniqueTriggerActivation.triggerUnique(unique, civInfo,
                         triggerNotificationText = "due to adopting [${belief.name}]")
 
         for (belief in beliefs)
-            for (unique in belief.uniqueObjects.filter { !it.hasTriggerConditional() })
-                UniqueTriggerActivation.triggerCivwideUnique(unique, civInfo)
+            for (unique in belief.uniqueObjects.filter { !it.hasTriggerConditional() && it.conditionalsApply(StateForConditionals(civInfo)) })
+                UniqueTriggerActivation.triggerUnique(unique, civInfo)
 
         civInfo.updateStatsForNextTurn()  // a belief can have an immediate effect on stats
     }
@@ -419,15 +414,12 @@ class ReligionManager : IsPartOfGameInfoSerialization {
         val newReligion = Religion(name, civInfo.gameInfo, civInfo.civName)
         newReligion.displayName = displayName
         if (religion != null) {
-            newReligion.followerBeliefs.addAll(religion!!.followerBeliefs)
-            newReligion.founderBeliefs.addAll(religion!!.founderBeliefs)
+            newReligion.addBeliefs(religion!!.getAllBeliefsOrdered().asIterable())
         }
 
         religion = newReligion
         civInfo.gameInfo.religions[name] = newReligion
 
-        religionState = ReligionState.Religion
-
         val holyCity = civInfo.cities.first { it.id == foundingCityId }
         holyCity.religion.religionThisIsTheHolyCityOf = newReligion.name
         holyCity.religion.addPressure(name, holyCity.population.population * 500)
@@ -443,7 +435,10 @@ class ReligionManager : IsPartOfGameInfoSerialization {
     fun maySpreadReligionAtAll(missionary: MapUnit): Boolean {
         if (!civInfo.isMajorCiv()) return false // Only major civs
         if (!civInfo.gameInfo.isReligionEnabled()) return false // No religion, no spreading
-        if (!missionary.canDoLimitedAction(Constants.spreadReligion)) return false
+
+        val religion = missionary.civ.gameInfo.religions[missionary.religion] ?: return false
+        if (religion.isPantheon()) return false
+        if (UnitActionModifiers.getUsableUnitActionUniques(missionary, UniqueType.CanSpreadReligion).none()) return false
         return true
     }
 
@@ -473,6 +468,26 @@ class ReligionManager : IsPartOfGameInfoSerialization {
         if (religion == null) return null
         return civInfo.gameInfo.getCities().firstOrNull { it.isHolyCityOf(religion!!.name) }
     }
+
+    fun getMajorityReligion(): Religion? {
+        // let's count for each religion (among those actually presents in civ's cities)
+        val religionCounter = Counter<Religion>()
+        for (city in civInfo.cities) {
+            // if city's majority Religion is null, let's just continue to next loop iteration
+            val cityMajorityReligion = city.religion.getMajorityReligion() ?: continue
+            // if not yet continued to next iteration from previous line, let's add the Religion to religionCounter
+            religionCounter.add(cityMajorityReligion, 1)
+        }
+        // let's get the max-counted Religion if there is one, null otherwise; if null, return null
+        val maxReligionCounterEntry = religionCounter.maxByOrNull { it.value } ?: return null
+        // if not returned null from prev. line, check if the maxReligion is in most of the cities
+        return if (maxReligionCounterEntry.value > civInfo.cities.size / 2)
+        // if maxReligionCounterEntry > half-cities-count we return the Religion of maxReligionCounterEntry
+            maxReligionCounterEntry.key
+        else
+        // if maxReligionCounterEntry <= half-cities-count we just return null
+            null
+    }
 }
 
 enum class ReligionState : IsPartOfGameInfoSerialization {
