@@ -7,6 +7,7 @@ import com.unciv.logic.civilization.diplomacy.RelationshipLevel
 import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.models.ruleset.Policy
 import com.unciv.models.ruleset.tile.ResourceType
+import com.unciv.models.ruleset.tile.TileImprovement
 import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.UniqueTarget
 import com.unciv.models.ruleset.unique.UniqueType
@@ -92,37 +93,34 @@ class CivInfoStatsForNextTurn(val civInfo: Civilization) {
                 .map { it.params[0] }.toHashSet() // needs to be .toHashSet()ed,
         // Because we go over every tile in every city and check if it's in this list, which can get real heavy.
 
+        fun addMaintenanceUniques(road: TileImprovement, type: UniqueType, state: StateForConditionals) {
+            for (unique in road.getMatchingUniques(type, state))
+                transportationUpkeep.add(Stat.valueOf(unique.params[1]), unique.params[0].toFloat())
+        }
+
         for (city in civInfo.cities) {
             for (tile in city.getTiles()) {
                 if (tile.isCityCenter()) continue
                 if (tile.getUnpillagedRoad() == RoadStatus.None) continue // Cheap checks before pricey checks
                 if (ignoredTileTypes.any { tile.matchesFilter(it, civInfo) }) continue
-                val road = tile.getUnpillagedRoadImprovement()
-                if (road!!.hasUnique(UniqueType.ImprovementMaintenance, StateForConditionals(civInfo, tile = tile))) {
-                    for(unique in road.getMatchingUniques(UniqueType.ImprovementMaintenance)) {
-                        transportationUpkeep.add(Stat.valueOf(unique.params[1]), unique.params[0].toFloat())
-                    }
-                }
-                if (road.hasUnique(UniqueType.ImprovementAllMaintenance, StateForConditionals(civInfo, tile = tile))) {
-                    for(unique in road.getMatchingUniques(UniqueType.ImprovementAllMaintenance)) {
-                        transportationUpkeep.add(Stat.valueOf(unique.params[1]), unique.params[0].toFloat())
-                    }
-                }
+                val road = tile.getUnpillagedRoadImprovement()!!  // covered by RoadStatus.None test
+                val stateForConditionals = StateForConditionals(civInfo, tile = tile)
+                addMaintenanceUniques(road, UniqueType.ImprovementMaintenance, stateForConditionals)
+                addMaintenanceUniques(road, UniqueType.ImprovementAllMaintenance, stateForConditionals)
             }
         }
+
         // tabulate neutral roads
         for (position in civInfo.neutralRoads) {
             val tile = civInfo.gameInfo.tileMap[position]
             if (tile.getUnpillagedRoad() == RoadStatus.None) continue // Cheap checks before pricey checks
-            val road = tile.getUnpillagedRoadImprovement()
-            if (road!!.hasUnique(UniqueType.ImprovementAllMaintenance, StateForConditionals(civInfo, tile = tile))) {
-                for(unique in road.getMatchingUniques(UniqueType.ImprovementAllMaintenance)) {
-                    transportationUpkeep.add(Stat.valueOf(unique.params[1]), unique.params[0].toFloat())
-                }
-            }
+            val road = tile.getUnpillagedRoadImprovement()!!  // covered by RoadStatus.None test
+            val stateForConditionals = StateForConditionals(civInfo, tile = tile)
+            addMaintenanceUniques(road, UniqueType.ImprovementAllMaintenance, stateForConditionals)
         }
+
         for (unique in civInfo.getMatchingUniques(UniqueType.RoadMaintenance))
-            transportationUpkeep.times(unique.params[0].toPercent())
+            transportationUpkeep.timesInPlace(unique.params[0].toPercent())
 
         return transportationUpkeep
     }
@@ -143,7 +141,8 @@ class CivInfoStatsForNextTurn(val civInfo: Civilization) {
     }
     fun getUnitSupplyFromCities(): Int {
         return civInfo.cities.size *
-            (civInfo.getDifficulty().unitSupplyPerCity + civInfo.getMatchingUniques(UniqueType.UnitSupplyPerCity).sumOf { it.params[0].toInt() })
+            (civInfo.getDifficulty().unitSupplyPerCity
+                    + civInfo.getMatchingUniques(UniqueType.UnitSupplyPerCity).sumOf { it.params[0].toInt() })
     }
     fun getUnitSupplyFromPop(): Int {
         var totalSupply = civInfo.cities.sumOf { it.population.population } * civInfo.gameInfo.ruleset.modOptions.constants.unitSupplyPerPopulation
@@ -170,8 +169,8 @@ class CivInfoStatsForNextTurn(val civInfo: Civilization) {
 
         //City-States bonuses
         for (otherCiv in civInfo.getKnownCivs()) {
-            if (!otherCiv.isCityState()) continue
-            if (!otherCiv.getDiplomacyManager(civInfo.civName).isRelationshipLevelEQ(RelationshipLevel.Ally))
+            if (!otherCiv.isCityState) continue
+            if (otherCiv.getDiplomacyManager(civInfo.civName)!!.relationshipIgnoreAfraid() != RelationshipLevel.Ally)
                 continue
             for (unique in civInfo.getMatchingUniques(UniqueType.CityStateStatPercent)) {
                 statMap.add(
@@ -219,7 +218,7 @@ class CivInfoStatsForNextTurn(val civInfo: Civilization) {
     fun getHappinessBreakdown(): HashMap<String, Float> {
         val statMap = HashMap<String, Float>()
 
-        fun HashMap<String, Float>.add(key:String, value: Float){
+        fun HashMap<String, Float>.add(key:String, value: Float) {
             if (!containsKey(key)) put(key, value)
             else put(key, value+get(key)!!)
         }
@@ -230,10 +229,10 @@ class CivInfoStatsForNextTurn(val civInfo: Civilization) {
         for (unique in civInfo.getMatchingUniques(UniqueType.BonusHappinessFromLuxury))
             happinessPerUniqueLuxury += unique.params[0].toInt()
 
-        val ownedLuxuries = civInfo.getCivResources().map { it.resource }
+        val ownedLuxuries = civInfo.getCivResourceSupply().map { it.resource }
             .filter { it.resourceType == ResourceType.Luxury }
 
-        val relevantLuxuries = civInfo.getCivResources().asSequence()
+        val relevantLuxuries = civInfo.getCivResourceSupply().asSequence()
             .map { it.resource }
             .count { it.resourceType == ResourceType.Luxury
                     && it.getMatchingUniques(UniqueType.ObsoleteWith)
@@ -244,8 +243,8 @@ class CivInfoStatsForNextTurn(val civInfo: Civilization) {
             civInfo.getMatchingUniques(UniqueType.CityStateLuxuryHappiness).sumOf { it.params[0].toInt() } / 100f
 
         val luxuriesProvidedByCityStates = civInfo.getKnownCivs().asSequence()
-            .filter { it.isCityState() && it.getAllyCiv() == civInfo.civName }
-            .flatMap { it.getCivResources().map { res -> res.resource } }
+            .filter { it.isCityState && it.getAllyCiv() == civInfo.civName }
+            .flatMap { it.getCivResourceSupply().map { res -> res.resource } }
             .distinct()
             .count { it.resourceType === ResourceType.Luxury && ownedLuxuries.contains(it) }
 
@@ -286,21 +285,23 @@ class CivInfoStatsForNextTurn(val civInfo: Civilization) {
     private fun getGlobalStatsFromUniques():StatMap {
         val statMap = StatMap()
         if (civInfo.religionManager.religion != null) {
-            for (unique in civInfo.religionManager.religion!!.getFounderUniques()) {
-                if (unique.isOfType(UniqueType.StatsFromGlobalCitiesFollowingReligion)) {
-                    statMap.add(
-                        "Religion",
-                        unique.stats * civInfo.religionManager.numberOfCitiesFollowingThisReligion()
-                    )
-                }
-                if (unique.isOfType(UniqueType.StatsFromGlobalFollowers))
-                    statMap.add(
-                        "Religion",
-                        unique.stats * civInfo.religionManager.numberOfFollowersFollowingThisReligion(
-                            unique.params[2]
-                        ).toFloat() / unique.params[1].toFloat()
-                    )
-            }
+            for (unique in civInfo.religionManager.religion!!.founderBeliefUniqueMap.getMatchingUniques(
+                UniqueType.StatsFromGlobalCitiesFollowingReligion, StateForConditionals(civInfo)
+            ))
+                statMap.add(
+                    "Religion",
+                    unique.stats * civInfo.religionManager.numberOfCitiesFollowingThisReligion()
+                )
+
+            for (unique in civInfo.religionManager.religion!!.founderBeliefUniqueMap.getMatchingUniques(
+                UniqueType.StatsFromGlobalFollowers, StateForConditionals(civInfo)
+            ))
+                statMap.add(
+                    "Religion",
+                    unique.stats * civInfo.religionManager.numberOfFollowersFollowingThisReligion(
+                        unique.params[2]
+                    ).toFloat() / unique.params[1].toFloat()
+                )
         }
 
         for (unique in civInfo.getMatchingUniques(UniqueType.StatsPerPolicies)) {
@@ -311,7 +312,12 @@ class CivInfoStatsForNextTurn(val civInfo: Civilization) {
 
         for (unique in civInfo.getMatchingUniques(UniqueType.Stats))
             if (unique.sourceObjectType != UniqueTarget.Building && unique.sourceObjectType != UniqueTarget.Wonder)
-                statMap.add(unique.sourceObjectType!!.name, unique.stats)
+                statMap.add(unique.getSourceNameForUser(), unique.stats)
+
+        for (unique in civInfo.getMatchingUniques(UniqueType.StatsPerStat)) {
+            val amount = civInfo.getStatReserve(Stat.valueOf(unique.params[2])) / unique.params[1].toInt()
+            statMap.add("Stats", unique.stats.times(amount))
+        }
 
         val statsPerNaturalWonder = Stats(happiness = 1f)
 
