@@ -82,10 +82,9 @@ class TileImprovementFunctions(val tile: Tile) {
                     .any { civInfo.getResourceAmount(it.params[1]) < it.params[0].toInt() })
             yield(ImprovementBuildingProblem.MissingResources)
 
-        val knownFeatureRemovals = tile.ruleset.tileRemovals
+        val knownFeatureRemovals = tile.ruleset.nonRoadTileRemovals
             .filter { rulesetImprovement ->
-                        RoadStatus.values().none { it.removeAction == rulesetImprovement.name }
-                        && (rulesetImprovement.techRequired == null || civInfo.tech.isResearched(rulesetImprovement.techRequired!!))
+                        rulesetImprovement.techRequired == null || civInfo.tech.isResearched(rulesetImprovement.techRequired!!)
             }
 
         if (!canImprovementBeBuiltHere(improvement, tile.hasViewableResource(civInfo), knownFeatureRemovals, stateForConditionals))
@@ -94,7 +93,7 @@ class TileImprovementFunctions(val tile: Tile) {
             yield(ImprovementBuildingProblem.Other)
     }
 
-    /** Without regards to what CivInfo it is, a lot of the checks are just for the improvement on the tile.
+    /** Without regards to what CivInfo it is (so no tech requirement check), a lot of the checks are just for the improvement on the tile.
      *  Doubles as a check for the map editor.
      */
     internal fun canImprovementBeBuiltHere(
@@ -105,7 +104,7 @@ class TileImprovementFunctions(val tile: Tile) {
         isNormalizeCheck: Boolean = false
     ): Boolean {
 
-        fun TileImprovement.canBeBuildOnThisUnbuildableTerrain(
+        fun TileImprovement.canBeBuiltOnThisUnbuildableTerrain(
             knownFeatureRemovals: List<TileImprovement>? = null,
         ): Boolean {
             val topTerrain = tile.lastTerrain
@@ -121,7 +120,7 @@ class TileImprovementFunctions(val tile: Tile) {
             if (featureRemovals.any { it !in knownFeatureRemovals }) return false
             val clonedTile = tile.clone()
             clonedTile.setTerrainFeatures(tile.terrainFeatures.filterNot {
-                feature -> featureRemovals.any{ it.name.removePrefix(Constants.remove) == feature } })
+                feature -> featureRemovals.any { it.name.removePrefix(Constants.remove) == feature } })
             return clonedTile.improvementFunctions.canImprovementBeBuiltHere(improvement, resourceIsVisible, knownFeatureRemovals, stateForConditionals)
         }
 
@@ -142,13 +141,13 @@ class TileImprovementFunctions(val tile: Tile) {
             RoadStatus.values().any { it.name == improvement.name } -> !tile.isWater
                     && RoadStatus.valueOf(improvement.name) > tile.roadStatus
 
-            // Then we check if there is any reason to not allow this improvement to be build
+            // Then we check if there is any reason to not allow this improvement to be built
 
             // Can't build if there is already an irremovable improvement here
             tile.improvement != null && tile.getTileImprovement()!!.hasUnique(UniqueType.Irremovable, stateForConditionals) -> false
 
             // Can't build if this terrain is unbuildable, except when we are specifically allowed to
-            tile.lastTerrain.unbuildable && !improvement.canBeBuildOnThisUnbuildableTerrain(knownFeatureRemovals) -> false
+            tile.lastTerrain.unbuildable && !improvement.canBeBuiltOnThisUnbuildableTerrain(knownFeatureRemovals) -> false
 
             // Can't build if any terrain specifically prevents building this improvement
             tile.getTerrainMatchingUniques(UniqueType.RestrictedBuildableImprovements, stateForConditionals).any {
@@ -157,14 +156,14 @@ class TileImprovementFunctions(val tile: Tile) {
 
             // Can't build if the improvement specifically prevents building on some present feature
             improvement.getMatchingUniques(UniqueType.CannotBuildOnTile, stateForConditionals).any {
-                    unique -> tile.matchesTerrainFilter(unique.params[0])
+                    unique -> tile.matchesFilter(unique.params[0], stateForConditionals.civInfo)
             } ->
                 false
 
             // Can't build if an improvement is only allowed to be built on specific tiles and this is not one of them
             // If multiple uniques of this type exists, we want all to match (e.g. Hill _and_ Forest would be meaningful)
             improvement.getMatchingUniques(UniqueType.CanOnlyBeBuiltOnTile, stateForConditionals).let {
-                it.any() && it.any { unique -> !tile.matchesTerrainFilter(unique.params[0]) }
+                it.any() && it.any { unique -> !tile.matchesFilter(unique.params[0], stateForConditionals.civInfo) }
             } -> false
 
             // Can't build if the improvement requires an adjacent terrain that is not present
@@ -356,8 +355,7 @@ class TileImprovementFunctions(val tile: Tile) {
             val otherCiv = tileToTakeOver.getOwner()
             if (otherCiv != null) {
                 // decrease relations for -10 pt/tile
-                if (!otherCiv.knows(civ)) otherCiv.diplomacyFunctions.makeCivilizationsMeet(civ)
-                otherCiv.getDiplomacyManager(civ).addModifier(DiplomaticModifiers.StealingTerritory, -10f)
+                otherCiv.getDiplomacyManagerOrMeet(civ).addModifier(DiplomaticModifiers.StealingTerritory, -10f)
                 civsToNotify.add(otherCiv)
             }
             nearestCity.expansion.takeOwnership(tileToTakeOver)
