@@ -12,7 +12,7 @@ import com.unciv.models.stats.Stat
 import com.unciv.models.stats.Stats
 import com.unciv.ui.components.extensions.toPercent
 
-fun Iterable<Pair<String, Stats>>.toStats(): Stats {
+fun List<Pair<String, Stats>>.toStats(): Stats {
     val stats = Stats()
     for ((_, statsToAdd) in this)
         stats.add(statsToAdd)
@@ -77,7 +77,7 @@ class TileStatFunctions(val tile: Tile) {
             for (unique in statsFromTilesUniques + statsFromObjectsUniques + statsFromTilesWithoutUniques) {
                 val tileType = unique.params[1]
                 if (tile.matchesFilter(tileType, observingCiv, true))
-                    listOfStats.add("{${unique.sourceObjectName}} ({${unique.text}})" to unique.stats)
+                    listOfStats.add("{${unique.sourceObjectName}} ({${unique.getDisplayText()}})" to unique.stats)
                 else if (improvement != null && improvement.matchesFilter(tileType))
                     improvementStats.add(unique.stats)
                 else if (road != null && road.matchesFilter(tileType))
@@ -147,7 +147,7 @@ class TileStatFunctions(val tile: Tile) {
         val list = arrayListOf(terrain.name to (terrain as Stats))
 
         for (unique in terrain.getMatchingUniques(UniqueType.Stats, stateForConditionals)) {
-            list.add(terrain.name+": "+unique.text to unique.stats)
+            list.add(terrain.name+": "+unique.getDisplayText() to unique.stats)
         }
         return list
     }
@@ -266,9 +266,12 @@ class TileStatFunctions(val tile: Tile) {
         improvement: TileImprovement,
         observingCiv: Civilization,
         city: City?,
-        cityUniqueCache: LocalUniqueCache = LocalUniqueCache(false)): Stats {
+        cityUniqueCache: LocalUniqueCache = LocalUniqueCache(false),
+        /** Provide this for performance */
+        currentTileStats: Stats? = null): Stats {
 
-        val currentStats = getTileStats(city, observingCiv, cityUniqueCache)
+        val currentStats = currentTileStats
+            ?: getTileStats(city, observingCiv, cityUniqueCache)
 
         val tileClone = tile.clone()
         tileClone.setTerrainTransients()
@@ -290,7 +293,7 @@ class TileStatFunctions(val tile: Tile) {
         if (tile.hasViewableResource(observingCiv) && tile.tileResource.isImprovedBy(improvement.name)
                 && tile.tileResource.improvementStats != null
         )
-            stats.add(tile.tileResource.improvementStats!!.clone()) // resource-specific improvement
+            stats.add(tile.tileResource.improvementStats!!) // resource-specific improvement
 
         val conditionalState = StateForConditionals(civInfo = observingCiv, city = city, tile = tile)
         for (unique in improvement.getMatchingUniques(UniqueType.Stats, conditionalState)) {
