@@ -9,7 +9,6 @@ import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.civilization.EspionageAction
 import com.unciv.logic.civilization.NotificationCategory
 import com.unciv.logic.civilization.NotificationIcon
-import com.unciv.logic.civilization.diplomacy.DiplomacyFunctions
 import com.unciv.logic.civilization.diplomacy.DiplomaticModifiers
 import com.unciv.logic.civilization.managers.EspionageManager
 import com.unciv.models.ruleset.unique.StateForConditionals
@@ -103,7 +102,7 @@ class Spy private constructor() : IsPartOfGameInfoSerialization {
             }
             SpyAction.EstablishNetwork -> {
                 val city = getCity() // This should never throw an exception, as going to the hideout sets your action to None.
-                if (city.civ.isCityState())
+                if (city.civ.isCityState)
                     setAction(SpyAction.RiggingElections, (getCity().civ.flagsCountdown[CivFlags.TurnsTillCityStateElection.name] ?: 1) - 1)
                 else if (city.civ == civInfo)
                     setAction(SpyAction.CounterIntelligence, 10)
@@ -239,11 +238,11 @@ class Spy private constructor() : IsPartOfGameInfoSerialization {
         } else startStealingTech()  // reset progress
 
         if (spyResult >= 100) {
-            otherCiv.getDiplomacyManager(civInfo).addModifier(DiplomaticModifiers.SpiedOnUs, -15f)
+            otherCiv.getDiplomacyManager(civInfo)?.addModifier(DiplomaticModifiers.SpiedOnUs, -15f)
         }
     }
 
-    fun canDoCoup(): Boolean = getCityOrNull() != null && getCity().civ.isCityState() && isSetUp() && getCity().civ.getAllyCiv() != civInfo.civName
+    fun canDoCoup(): Boolean = getCityOrNull() != null && getCity().civ.isCityState && isSetUp() && getCity().civ.getAllyCiv() != civInfo.civName
 
     /**
      * Initiates a coup if this spies civ is not the ally of the city-state.
@@ -259,29 +258,28 @@ class Spy private constructor() : IsPartOfGameInfoSerialization {
         }
         val cityState = getCity().civ
         val allyCiv = cityState.getAllyCiv()?.let { civInfo.gameInfo.getCivilization(it) }
-        if (allyCiv?.knows(civInfo) == false) civInfo.diplomacyFunctions.makeCivilizationsMeet(allyCiv)
 
         val successChance = getCoupChanceOfSuccess(true)
         val randomValue = Random(randomSeed()).nextFloat()
         if (randomValue <= successChance) {
             // Success
-            val previousInfluence = if (allyCiv != null) cityState.getDiplomacyManager(allyCiv).getInfluence() else 80f
-            cityState.getDiplomacyManager(civInfo).setInfluence(previousInfluence)
+            val previousInfluence = if (allyCiv != null) cityState.getDiplomacyManager(allyCiv)!!.getInfluence() else 80f
+            cityState.getDiplomacyManager(civInfo)!!.setInfluence(previousInfluence)
 
             civInfo.addNotification("Your spy [$name] successfully staged a coup in [${cityState.civName}]!", getCity().location,
                     NotificationCategory.Espionage, NotificationIcon.Spy, cityState.civName)
             if (allyCiv != null) {
-                cityState.getDiplomacyManager(allyCiv).reduceInfluence(20f)
+                cityState.getDiplomacyManagerOrMeet(allyCiv).reduceInfluence(20f)
                 allyCiv.addNotification("A spy from [${civInfo.civName}] successfully staged a coup in our former ally [${cityState.civName}]!", getCity().location,
                         NotificationCategory.Espionage, civInfo.civName,  NotificationIcon.Spy, cityState.civName)
-                allyCiv.getDiplomacyManager(civInfo).addModifier(DiplomaticModifiers.SpiedOnUs, -15f)
+                allyCiv.getDiplomacyManagerOrMeet(civInfo).addModifier(DiplomaticModifiers.SpiedOnUs, -15f)
             }
             for (civ in cityState.getKnownCivsWithSpectators()) {
                 if (civ == allyCiv || civ == civInfo) continue
                 civ.addNotification("A spy from [${civInfo.civName}] successfully staged a coup in [${cityState.civName}]!", getCity().location,
                         NotificationCategory.Espionage, civInfo.civName,  NotificationIcon.Spy, cityState.civName)
                 if (civ.isSpectator()) continue
-                cityState.getDiplomacyManager(civ).reduceInfluence(10f) // Guess
+                cityState.getDiplomacyManager(civ)!!.reduceInfluence(10f) // Guess
             }
             setAction(SpyAction.RiggingElections, 10)
             cityState.cityStateFunctions.updateAllyCivForCityState()
@@ -289,10 +287,10 @@ class Spy private constructor() : IsPartOfGameInfoSerialization {
         } else {
             // Failure
             val spy = allyCiv?.espionageManager?.getSpyAssignedToCity(getCity())
-            cityState.getDiplomacyManager(civInfo).addInfluence(-20f)
+            cityState.getDiplomacyManager(civInfo)!!.addInfluence(-20f)
             allyCiv?.addNotification("A spy from [${civInfo.civName}] failed to stag a coup in our ally [${cityState.civName}] and was killed!", getCity().location,
                     NotificationCategory.Espionage, civInfo.civName,  NotificationIcon.Spy, cityState.civName)
-            allyCiv?.getDiplomacyManager(civInfo)?.addModifier(DiplomaticModifiers.SpiedOnUs, -10f)
+            allyCiv?.getDiplomacyManagerOrMeet(civInfo)?.addModifier(DiplomaticModifiers.SpiedOnUs, -10f)
 
             civInfo.addNotification("Our spy [$name] failed to stag a coup in [${cityState.civName}] and was killed!", getCity().location,
                     NotificationCategory.Espionage, civInfo.civName,  NotificationIcon.Spy, cityState.civName)
@@ -311,9 +309,9 @@ class Spy private constructor() : IsPartOfGameInfoSerialization {
 
         // Influence difference should always be a positive value
         var influenceDifference: Float = if (cityState.getAllyCiv() != null)
-            cityState.getDiplomacyManager(cityState.getAllyCiv()!!).getInfluence()
+            cityState.getDiplomacyManager(cityState.getAllyCiv()!!)!!.getInfluence()
         else 60f
-        influenceDifference -= cityState.getDiplomacyManager(civInfo).getInfluence()
+        influenceDifference -= cityState.getDiplomacyManager(civInfo)!!.getInfluence()
         successPercentage -= influenceDifference / 2f
 
         // If we are viewing the success chance we don't want to reveal that there is a defending spy
@@ -389,12 +387,12 @@ class Spy private constructor() : IsPartOfGameInfoSerialization {
             city == null -> {
                 // Spy is in hideout - effectiveness won't matter
                 friendlyUniques = civInfo.getMatchingUniques(UniqueType.SpyEffectiveness)
-                enemyUniques = sequenceOf()
+                enemyUniques = emptySequence()
             }
             city.civ == civInfo -> {
                 // Spy is in our own city
                 friendlyUniques = city.getMatchingUniques(UniqueType.SpyEffectiveness, StateForConditionals(city), includeCivUniques = true)
-                enemyUniques = sequenceOf()
+                enemyUniques = emptySequence()
             }
             else -> {
                 // Spy is active in a foreign city
