@@ -2,8 +2,10 @@ package com.unciv.models.metadata
 
 import com.badlogic.gdx.Gdx
 import com.unciv.json.json
-import com.unciv.ui.screens.newgamescreen.TranslatedSelectBox
-import com.unciv.ui.screens.pickerscreens.Github
+import com.unciv.logic.github.GithubAPI
+import com.unciv.ui.components.widgets.TranslatedSelectBox
+import com.unciv.logic.github.Github
+import com.unciv.models.translations.tr
 
 
 class ModCategories : ArrayList<ModCategories.Category>() {
@@ -21,12 +23,12 @@ class ModCategories : ArrayList<ModCategories.Category>() {
         constructor() :
             this("", "", false, "", "")
 
-        constructor(topic: Github.TopicSearchResponse.Topic) :
+        constructor(topic: GithubAPI.TopicSearchResponse.Topic) :
             this(labelSuggestion(topic), topic.name, true, topic.created_at, topic.updated_at)
 
         companion object {
             val All = Category("All mods", "unciv-mod", false, "", "")
-            fun labelSuggestion(topic: Github.TopicSearchResponse.Topic) =
+            fun labelSuggestion(topic: GithubAPI.TopicSearchResponse.Topic) =
                 topic.display_name?.takeUnless { it.isBlank() }
                     ?: topic.name.removePrefix("unciv-mod-").replaceFirstChar(Char::titlecase)
         }
