@@ -3,14 +3,18 @@ package com.unciv.models.translations
 import com.badlogic.gdx.Gdx
 import com.unciv.Constants
 import com.unciv.UncivGame
+import com.unciv.models.metadata.GameSettings.LocaleCode
 import com.unciv.models.ruleset.RulesetCache
 import com.unciv.models.ruleset.unique.Unique
 import com.unciv.models.stats.Stat
 import com.unciv.models.stats.Stats
-import com.unciv.ui.components.Fonts
+import com.unciv.ui.components.fonts.DiacriticSupport
+import com.unciv.ui.components.fonts.FontRulesetIcons
 import com.unciv.utils.Log
 import com.unciv.utils.debug
 import java.util.Locale
+import org.jetbrains.annotations.VisibleForTesting
+import java.text.NumberFormat
 
 /**
  *  This collection holds all translations for the game.
@@ -31,7 +35,7 @@ import java.util.Locale
  *
  *  @see    String.tr   for more explanations (below)
  */
-class Translations : LinkedHashMap<String, TranslationEntry>(){
+class Translations : LinkedHashMap<String, TranslationEntry>() {
 
     var percentCompleteOfLanguages = HashMap<String,Int>()
             .apply { put(Constants.english, 100) } // So even if we don't manage to load the percentages, we can still pass the language screen
@@ -41,7 +45,6 @@ class Translations : LinkedHashMap<String, TranslationEntry>(){
     // used by tr() whenever GameInfo not initialized (allowing new game screen to use mod translations)
     var translationActiveMods = LinkedHashSet<String>()
 
-
     /**
      * Searches for the translation entry of a given [text] for a given [language].
      * This includes translations provided by mods from [activeMods]
@@ -81,7 +84,7 @@ class Translations : LinkedHashMap<String, TranslationEntry>(){
     /** This reads all translations for a specific language, including _all_ installed mods.
      *  Vanilla translations go into `this` instance, mod translations into [modsWithTranslations].
      */
-    private fun tryReadTranslationForLanguage(language: String) {
+    private fun tryReadTranslationForLanguage(language: String, noDiacritics: Boolean = false) {
         val translationStart = System.currentTimeMillis()
 
         val translationFileName = "jsons/translations/$language.properties"
@@ -106,34 +109,40 @@ class Translations : LinkedHashMap<String, TranslationEntry>(){
                     modsWithTranslations[modFolder.name()] = translationsForMod
                 }
                 try {
-                    translationsForMod.createTranslations(language, TranslationFileReader.read(modTranslationFile))
+                    translationsForMod.createTranslations(language, TranslationFileReader.read(modTranslationFile), noDiacritics)
                 } catch (ex: Exception) {
                     Log.error("Exception reading translations for ${modFolder.name()} $language", ex)
                 }
             }
         }
 
-        createTranslations(language, languageTranslations)
+        createTranslations(language, languageTranslations, noDiacritics)
 
         debug("Loading translation file for %s - %sms", language, System.currentTimeMillis() - translationStart)
     }
 
-    private fun createTranslations(language: String, languageTranslations: HashMap<String,String>) {
-        for (translation in languageTranslations) {
-            val hashKey = if (translation.key.contains('[') && !translation.key.contains('<'))
-                translation.key.getPlaceholderText()
-            else translation.key
+    @VisibleForTesting
+    fun createTranslations(language: String, languageTranslations: HashMap<String, String>, noDiacritics: Boolean = false) {
+        val diacriticSupport = if (noDiacritics) null
+            else DiacriticSupport(languageTranslations).takeIf { it.isEnabled() }
+        for ((key, value) in languageTranslations) {
+            val hashKey = if (key.contains('[') && !key.contains('<'))
+                key.getPlaceholderText()
+            else key
             var entry = this[hashKey]
             if (entry == null) {
-                entry = TranslationEntry(translation.key)
+                entry = TranslationEntry(key)
                 this[hashKey] = entry
             }
-            entry[language] = translation.value
+            entry[language] = diacriticSupport?.remapDiacritics(value) ?: value
         }
     }
 
-    fun tryReadTranslationForCurrentLanguage(){
+
+    fun tryReadTranslationForCurrentLanguage() {
+        DiacriticSupport.reset()
         tryReadTranslationForLanguage(UncivGame.Current.settings.language)
+        DiacriticSupport.freeTranslationData()
     }
 
     /** Get a list of supported languages for [readAllLanguagesTranslation] */
@@ -147,7 +156,7 @@ class Translations : LinkedHashMap<String, TranslationEntry>(){
             for (file in Gdx.files.internal("jsons/translations").list())
                 languages.add(file.nameWithoutExtension())
         }
-        catch (ex:Exception) {
+        catch (ex: Exception) {
             Log.error("Failed to add languages", ex)
         } // Iterating on internal files will not work when running from a .jar
 
@@ -166,7 +175,15 @@ class Translations : LinkedHashMap<String, TranslationEntry>(){
         return languages.filter { Gdx.files.internal("jsons/translations/$it.properties").exists() }
     }
 
-    /** Ensure _all_ languages are loaded, used by [TranslationFileWriter] and `TranslationTests` */
+    /** Ensure _all_ languages are loaded, used by [TranslationFileWriter] and `TranslationTests` only.
+     *
+     *  #### Notes:
+     *  -  Expects to run on a newly created instance.
+     *  -  Loads the translations with no diacritic mapping, so what we read will be what we write
+     *     (otherwise we would write out the fake alphabet-conversions, a one-way destructive mistake).
+     *  -  Relies on usage by TFW and tests only, if the result is ever meant to support translations that are actually displayed, a refactor will be needed.
+     *  -  Does not clear the fake alphabet possibly present in DiacriticSupport, but will not use it either.
+     */
     fun readAllLanguagesTranslation() {
         // Apparently you can't iterate over the files in a directory when running out of a .jar...
         // https://www.badlogicgames.com/forum/viewtopic.php?f=11&t=27250
@@ -175,13 +192,13 @@ class Translations : LinkedHashMap<String, TranslationEntry>(){
         val translationStart = System.currentTimeMillis()
 
         for (language in getLanguagesWithTranslationFile()) {
-            tryReadTranslationForLanguage(language)
+            tryReadTranslationForLanguage(language, noDiacritics = true)
         }
 
         debug("Loading translation files - %sms", System.currentTimeMillis() - translationStart)
     }
 
-    fun loadPercentageCompleteOfLanguages(){
+    fun loadPercentageCompleteOfLanguages() {
         val startTime = System.currentTimeMillis()
 
         percentCompleteOfLanguages = TranslationFileReader.readLanguagePercentages()
@@ -223,6 +240,26 @@ class Translations : LinkedHashMap<String, TranslationEntry>(){
         const val conditionalUniqueOrderString = "ConditionalsPlacement"
         const val shouldCapitalizeString = "StartWithCapitalLetter"
         const val effectBeforeCause = "EffectBeforeCause"
+
+        // NumberFormat cache, key: language, value: NumberFormat
+        private val languageToNumberFormat = mutableMapOf<String, NumberFormat>()
+
+        fun getLocaleFromLanguage(language: String): Locale {
+            val bannedCharacters =
+                listOf(' ', '_', '-', '(', ')') // Things not to have in enum names
+            val languageName = language.filterNot { it in bannedCharacters }
+            return try {
+                val code = LocaleCode.valueOf(languageName)
+                Locale(code.language, code.country)
+            } catch (_: Exception) {
+                Locale.getDefault()
+            }
+        }
+
+        fun getNumberFormatFromLanguage(language: String): NumberFormat =
+            languageToNumberFormat.getOrPut(language) {
+                NumberFormat.getInstance(getLocaleFromLanguage(language))
+            }
     }
 }
 
@@ -244,6 +281,9 @@ val curlyBraceRegex = Regex("""\{([^}]*)\}""")
 @Suppress("RegExpRedundantEscape") // Some Android versions need ]}) escaped
 val pointyBraceRegex = Regex("""\<([^>]*)\>""")
 
+// Used to match continous digits 0, 12, 1232 etc
+@Suppress("RegExpRedundantEscape") // Some Android versions need ]}) escaped
+val digitsRegex = Regex("""\d+""")
 
 object TranslationActiveModsCache {
     private var cachedHash = Int.MIN_VALUE
@@ -283,7 +323,7 @@ object TranslationActiveModsCache {
     }
 }
 
-    /**
+/**
  *  This function does the actual translation work,
  *      using an instance of [Translations] stored in UncivGame.Current
  *
@@ -293,141 +333,157 @@ object TranslationActiveModsCache {
  *                  sentences - contains at least one '{'
  *                  - phrases between curly braces are translated individually
  *                  Additionally, they may contain conditionals between '<' and '>'
+ *  @param      hideIcons disables auto-inserting icons for ruleset objects (but not Stats)
  *  @return     The translated string
  *                  defaults to the input string if no translation is available,
  *                  but with placeholder or sentence brackets removed.
  */
-fun String.tr(hideIcons:Boolean = false): String {
-    val language:String = UncivGame.Current.settings.language
+fun String.tr(hideIcons: Boolean = false): String {
+    val language: String = UncivGame.Current.settings.language
 
     // '<' and '>' checks for quick 'no' answer, regex to ensure that no one accidentally put '><' and ruined things
-    if (contains('<') && contains('>') && pointyBraceRegex.containsMatchIn(this)) { // Conditionals!
-        /**
-         * So conditionals can contain placeholders, such as <vs [unitFilter] units>, which themselves
-         * can contain multiple filters, such as <vs [{Military} {Water}] units>.
-         * Moreover, we can have any amount of conditionals in any order, and translations
-         * can reorder these conditionals in any way they like, even putting them in front
-         * of the rest of the translatable string.
-         * All of this nesting makes it quite difficult to translate, and is the reason we check
-         * for these first.
-         *
-         * The plan: First translate each of the conditionals on its own, and then combine them
-         * together into the final fully translated string.
-         */
-
-        var translatedBaseUnique = this.removeConditionals().tr(hideIcons)
-
-        val conditionals = this.getConditionals().map { it.placeholderText }
-        val conditionsWithTranslation: LinkedHashMap<String, String> = linkedMapOf()
-
-        for (conditional in this.getConditionals())
-            conditionsWithTranslation[conditional.placeholderText] = conditional.text.tr(hideIcons)
-
-        val translatedConditionals: MutableList<String> = mutableListOf()
-
-        // Somewhere, we asked the translators to reorder all possible conditionals in a way that
-        // makes sense in their language. We get this ordering, and than extract each of the
-        // translated conditionals, removing the <> surrounding them, and removing param values
-        // where it exists.
-        val conditionalOrdering = UncivGame.Current.translations.getConditionalOrder(language)
-        for (placedConditional in pointyBraceRegex.findAll(conditionalOrdering)
-            .map { it.value.substring(1, it.value.length - 1).getPlaceholderText() }) {
-            if (placedConditional in conditionals) {
-                translatedConditionals.add(conditionsWithTranslation[placedConditional]!!)
-                conditionsWithTranslation.remove(placedConditional)
-            }
-        }
-
-        // If the translated string that should contain all conditionals doesn't contain
-        // a few conditionals used here, just add the translations of these to the end.
-        // We do test for this, but just in case.
-        translatedConditionals.addAll(conditionsWithTranslation.values)
-
-        // After that, add the translation of the base unique either before or after these conditionals
-        if (UncivGame.Current.translations.placeConditionalsAfterUnique(language)) {
-            translatedConditionals.add(0, translatedBaseUnique)
-        } else {
-            if (UncivGame.Current.translations.shouldCapitalize(language) && translatedBaseUnique[0].isUpperCase())
-                translatedBaseUnique = translatedBaseUnique.replaceFirstChar { it.lowercase() }
-            translatedConditionals.add(translatedBaseUnique)
-        }
-
-        var fullyTranslatedString = translatedConditionals.joinToString(
-            UncivGame.Current.translations.getSpaceEquivalent(language)
-        )
-        if (UncivGame.Current.translations.shouldCapitalize(language))
-            fullyTranslatedString = fullyTranslatedString.replaceFirstChar { it.uppercase() }
-        return fullyTranslatedString
+    if (contains('<') && contains('>') && pointyBraceRegex.containsMatchIn(this)) {
+        return translateConditionals(hideIcons, language)
     }
 
     // curly and square brackets can be nested inside of each other so find the leftmost curly/square
     // bracket then process that first
     val indexSquare = this.indexOf('[')
     val indexCurly = this.indexOf('{')
-    val processSquare = indexSquare >= 0 && (indexCurly < 0 || indexSquare < indexCurly)
-    val processCurly =  indexCurly >= 0 && (indexSquare < 0 || indexCurly < indexSquare)
 
-    // There might still be optimization potential here!
-    if (processSquare) { // Placeholders!
-        /**
-         * I'm SURE there's an easier way to do this but I can't think of it =\
-         * So what's all this then?
-         * Well, not all languages are like English. So say I want to say "work on Library has completed in Akkad",
-         * but in a completely different language like Japanese or German,
-         * It could come out "Akkad hast die worken onner Library gerfinishen" or whatever,
-         * basically, the order of the words in the sentence is not guaranteed.
-         * So to translate this, I give a sentence like "work on [building] has completed in [city]"
-         * and the german can put those placeholders where he wants, so  "[city] hast die worken onner [building] gerfinishen"
-         * The string on which we call tr() will look like "work on [library] has completed in [Akkad]"
-         * We will find the german placeholder text, and replace the placeholders with what was filled in the text we got!
-         */
+    val squareBracketsEncounteredFirst = indexSquare >= 0 && (indexCurly < 0 || indexSquare < indexCurly)
+    val curlyBracketsEncounteredFirst =  indexCurly >= 0 && (indexSquare < 0 || indexCurly < indexSquare)
 
-        // Convert "work on [building] has completed in [city]" to "work on [] has completed in []"
-        val translationStringWithSquareBracketsOnly = this.getPlaceholderText()
-        // That is now the key into the translation HashMap!
-        val translationEntry = UncivGame.Current.translations
-            .get(translationStringWithSquareBracketsOnly, language, TranslationActiveModsCache.activeMods)
+    if (squareBracketsEncounteredFirst)
+        return translatePlaceholders(language, hideIcons)
 
-        var languageSpecificPlaceholder: String
-        val originalEntry: String
-        if (translationEntry == null || !translationEntry.containsKey(language)) {
-            // Translation placeholder doesn't exist for this language, default to English
-            languageSpecificPlaceholder = this
-            originalEntry = this
-        } else {
-            languageSpecificPlaceholder = translationEntry[language]!!
-            originalEntry = translationEntry.entry
-        }
-
-        // Take the terms in the message, WITHOUT square brackets
-        val termsInMessage = this.getPlaceholderParametersIgnoringLowerLevelBraces()
-        // Take the terms from the placeholder
-        val termsInTranslationPlaceholder = originalEntry.getPlaceholderParametersIgnoringLowerLevelBraces()
-        if (termsInMessage.size != termsInTranslationPlaceholder.size)
-            throw Exception("Message $this has a different number of terms than the placeholder $translationEntry!")
-
-        for (i in termsInMessage.indices) {
-            languageSpecificPlaceholder = languageSpecificPlaceholder.replace(
-                "[${termsInTranslationPlaceholder[i]}]", // re-add square brackets to placeholder terms
-                termsInMessage[i].tr(hideIcons)
-            )
-        }
-        return languageSpecificPlaceholder      // every component is already translated
-    }
-
-    if (processCurly) { // Translating partial sentences
+    if (curlyBracketsEncounteredFirst) // Translating partial sentences
         return curlyBraceRegex.replace(this) { it.groups[1]!!.value.tr(hideIcons) }
+
+    return translateIndividualWord(language, hideIcons)
+}
+
+
+private fun String.translateConditionals(hideIcons: Boolean, language: String): String {
+    /**
+     * So conditionals can contain placeholders, such as <vs [unitFilter] units>, which themselves
+     * can contain multiple filters, such as <vs [{Military} {Water}] units>.
+     * Moreover, we can have any amount of conditionals in any order, and translations
+     * can reorder these conditionals in any way they like, even putting them in front
+     * of the rest of the translatable string.
+     * All of this nesting makes it quite difficult to translate, and is the reason we check
+     * for these first.
+     *
+     * The plan: First translate each of the conditionals on its own, and then combine them
+     * together into the final fully translated string.
+     */
+
+    var translatedBaseUnique = this.removeConditionals().tr(hideIcons)
+
+    val conditionals = this.getConditionals().map { it.placeholderText }
+    val conditionsWithTranslation: LinkedHashMap<String, String> = linkedMapOf()
+
+    for (conditional in this.getConditionals())
+        conditionsWithTranslation[conditional.placeholderText] = conditional.text.tr(hideIcons)
+
+    val translatedConditionals: MutableList<String> = mutableListOf()
+
+    // Somewhere, we asked the translators to reorder all possible conditionals in a way that
+    // makes sense in their language. We get this ordering, and than extract each of the
+    // translated conditionals, removing the <> surrounding them, and removing param values
+    // where it exists.
+    val conditionalOrdering = UncivGame.Current.translations.getConditionalOrder(language)
+    for (placedConditional in pointyBraceRegex.findAll(conditionalOrdering)
+        .map { it.value.substring(1, it.value.length - 1).getPlaceholderText() }) {
+        if (placedConditional in conditionals) {
+            translatedConditionals.add(conditionsWithTranslation[placedConditional]!!)
+            conditionsWithTranslation.remove(placedConditional)
+        }
     }
 
+    // If the translated string that should contain all conditionals doesn't contain
+    // a few conditionals used here, just add the translations of these to the end.
+    // We do test for this, but just in case.
+    translatedConditionals.addAll(conditionsWithTranslation.values)
+
+    // After that, add the translation of the base unique either before or after these conditionals
+    if (UncivGame.Current.translations.placeConditionalsAfterUnique(language)) {
+        translatedConditionals.add(0, translatedBaseUnique)
+    } else {
+        if (UncivGame.Current.translations.shouldCapitalize(language) && translatedBaseUnique[0].isUpperCase())
+            translatedBaseUnique = translatedBaseUnique.replaceFirstChar { it.lowercase() }
+        translatedConditionals.add(translatedBaseUnique)
+    }
+
+    var fullyTranslatedString = translatedConditionals.joinToString(
+        UncivGame.Current.translations.getSpaceEquivalent(language)
+    )
+    if (UncivGame.Current.translations.shouldCapitalize(language))
+        fullyTranslatedString = fullyTranslatedString.replaceFirstChar { it.uppercase() }
+    return fullyTranslatedString
+}
+
+private fun String.translatePlaceholders(language: String, hideIcons: Boolean): String {
+    /**
+     * I'm SURE there's an easier way to do this but I can't think of it =\
+     * So what's all this then?
+     * Well, not all languages are like English. So say I want to say "work on Library has completed in Akkad",
+     * but in a completely different language like Japanese or German,
+     * It could come out "Akkad hast die worken onner Library gerfinishen" or whatever,
+     * basically, the order of the words in the sentence is not guaranteed.
+     * So to translate this, I give a sentence like "work on [building] has completed in [city]"
+     * and the german can put those placeholders where he wants, so  "[city] hast die worken onner [building] gerfinishen"
+     * The string on which we call tr() will look like "work on [library] has completed in [Akkad]"
+     * We will find the german placeholder text, and replace the placeholders with what was filled in the text we got!
+     */
+
+    // Convert "work on [building] has completed in [city]" to "work on [] has completed in []"
+    val translationStringWithSquareBracketsOnly = this.getPlaceholderText()
+    // That is now the key into the translation HashMap!
+    val translationEntry = UncivGame.Current.translations
+        .get(translationStringWithSquareBracketsOnly, language, TranslationActiveModsCache.activeMods)
+
+    var languageSpecificPlaceholder: String
+    val originalEntry: String
+    if (translationEntry == null || !translationEntry.containsKey(language)) {
+        // Translation placeholder doesn't exist for this language, default to English
+        languageSpecificPlaceholder = this
+        originalEntry = this
+    } else {
+        languageSpecificPlaceholder = translationEntry[language]!!
+        originalEntry = translationEntry.entry
+    }
+
+    // Take the terms in the message, WITHOUT square brackets
+    val termsInMessage = this.getPlaceholderParameters()
+    // Take the terms from the placeholder
+    val termsInTranslationPlaceholder = originalEntry.getPlaceholderParameters()
+    if (termsInMessage.size != termsInTranslationPlaceholder.size)
+        throw Exception("Message $this has a different number of terms than the placeholder $translationEntry!")
+
+    for (i in termsInMessage.indices) {
+        languageSpecificPlaceholder = languageSpecificPlaceholder.replace(
+            "[${termsInTranslationPlaceholder[i]}]", // re-add square brackets to placeholder terms
+            termsInMessage[i].tr(hideIcons)
+        )
+    }
+    return languageSpecificPlaceholder      // every component is already translated
+}
+
+
+/** No brackets of any kind, just a single word */
+private fun String.translateIndividualWord(language: String, hideIcons: Boolean): String {
     if (Stats.isStats(this)) return Stats.parse(this).toString()
 
-    val translation = UncivGame.Current.translations.getText(this, language, TranslationActiveModsCache.activeMods)
+    val translation = UncivGame.Current.translations.getText(
+        this, language, TranslationActiveModsCache.activeMods
+    ).replace(digitsRegex) { it.value.toLong().tr(language) }
 
     val stat = Stat.safeValueOf(this)
     if (stat != null) return stat.character + translation
 
-    if (!hideIcons && Fonts.rulesetObjectNameToChar.containsKey(this))
-        return Fonts.rulesetObjectNameToChar[this]!! + translation
+    if (!hideIcons && FontRulesetIcons.rulesetObjectNameToChar.containsKey(this))
+        return FontRulesetIcons.rulesetObjectNameToChar[this]!! + translation
 
     return translation
 }
@@ -438,17 +494,19 @@ fun String.tr(hideIcons:Boolean = false): String {
  * For example, a string like 'The city of [New [York]]' will return ['New [York]'],
  * allowing us to have nested translations!
  */
-fun String.getPlaceholderParametersIgnoringLowerLevelBraces(): List<String> {
+fun String.getPlaceholderParameters(): List<String> {
     if (!this.contains('[')) return emptyList()
+
+    val stringToParse = this.removeConditionals()
     val parameters = ArrayList<String>()
     var depthOfBraces = 0
     var startOfCurrentParameter = -1
-    for (i in this.indices) {
-        if (this[i] == '[') {
+    for (i in stringToParse.indices) {
+        if (stringToParse[i] == '[') {
             if (depthOfBraces == 0) startOfCurrentParameter = i+1
             depthOfBraces++
         }
-        if (this[i] == ']' && depthOfBraces > 0) {
+        if (stringToParse[i] == ']' && depthOfBraces > 0) {
             depthOfBraces--
             if (depthOfBraces == 0) parameters.add(substring(startOfCurrentParameter,i))
         }
@@ -458,13 +516,13 @@ fun String.getPlaceholderParametersIgnoringLowerLevelBraces(): List<String> {
 
 fun String.getPlaceholderText(): String {
     var stringToReturn = this.removeConditionals()
-    val placeholderParameters = stringToReturn.getPlaceholderParametersIgnoringLowerLevelBraces()
+    val placeholderParameters = stringToReturn.getPlaceholderParameters()
     for (placeholderParameter in placeholderParameters)
         stringToReturn = stringToReturn.replace("[$placeholderParameter]", "[]")
     return stringToReturn
 }
 
-fun String.equalsPlaceholderText(str:String): Boolean {
+fun String.equalsPlaceholderText(str: String): Boolean {
     if (first() != str.first()) return false // for quick negative return 95% of the time
     return this.getPlaceholderText() == str
 }
@@ -474,11 +532,6 @@ fun String.hasPlaceholderParameters(): Boolean {
     return squareBraceRegex.containsMatchIn(this.removeConditionals())
 }
 
-fun String.getPlaceholderParameters(): List<String> {
-    if (!this.contains('[')) return emptyList()
-    return squareBraceRegex.findAll(this.removeConditionals()).map { it.groups[1]!!.value }.toList()
-}
-
 /** Substitutes placeholders with [strings], respecting order of appearance. */
 fun String.fillPlaceholders(vararg strings: String): String {
     val keys = this.getPlaceholderParameters()
@@ -509,3 +562,13 @@ fun String.removeConditionals(): String {
         .replace("  ", " ")
         .trim()
 }
+
+// formats number according to current language
+fun Number.tr(): String {
+    return UncivGame.Current.settings.getCurrentNumberFormat().format(this)
+}
+
+// formats number according to given language
+fun Number.tr(language: String): String {
+    return Translations.getNumberFormatFromLanguage(language).format(this)
+}
