@@ -2,6 +2,10 @@ package com.unciv.ui.components.extensions
 
 import com.badlogic.gdx.Input
 import com.badlogic.gdx.graphics.Color
+import com.badlogic.gdx.graphics.Pixmap
+import com.badlogic.gdx.graphics.TextureData
+import com.badlogic.gdx.graphics.glutils.FileTextureData
+import com.badlogic.gdx.graphics.glutils.PixmapTextureData
 import com.badlogic.gdx.math.Rectangle
 import com.badlogic.gdx.math.Vector2
 import com.badlogic.gdx.scenes.scene2d.Actor
@@ -15,6 +19,7 @@ import com.badlogic.gdx.scenes.scene2d.ui.CheckBox
 import com.badlogic.gdx.scenes.scene2d.ui.Image
 import com.badlogic.gdx.scenes.scene2d.ui.ImageButton
 import com.badlogic.gdx.scenes.scene2d.ui.Label
+import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane
 import com.badlogic.gdx.scenes.scene2d.ui.Table
 import com.badlogic.gdx.scenes.scene2d.ui.TextButton
 import com.badlogic.gdx.scenes.scene2d.ui.TextButton.TextButtonStyle
@@ -22,10 +27,13 @@ import com.badlogic.gdx.scenes.scene2d.ui.WidgetGroup
 import com.badlogic.gdx.utils.Align
 import com.unciv.Constants
 import com.unciv.models.translations.tr
-import com.unciv.ui.components.Fonts
 import com.unciv.ui.components.extensions.GdxKeyCodeFixes.DEL
 import com.unciv.ui.components.extensions.GdxKeyCodeFixes.toString
 import com.unciv.ui.components.extensions.GdxKeyCodeFixes.valueOf
+import com.unciv.ui.components.fonts.Fonts
+import com.unciv.ui.components.input.KeyCharAndCode
+import com.unciv.ui.components.input.keyShortcuts
+import com.unciv.ui.components.input.onActivation
 import com.unciv.ui.components.input.onChange
 import com.unciv.ui.images.IconCircleGroup
 import com.unciv.ui.images.ImageGetter
@@ -40,6 +48,7 @@ private class RestorableTextButtonStyle(
     val restoreStyle: ButtonStyle
 ) : TextButtonStyle(baseStyle)
 
+//todo ButtonStyle *does* have a `disabled` Drawable, and Button ignores touches in disabled state anyway - all this is a wrong approach
 /** Disable a [Button] by setting its [touchable][Button.touchable] and [style][Button.style] properties. */
 fun Button.disable() {
     touchable = Touchable.disabled
@@ -97,15 +106,19 @@ fun Actor.centerY(parent: Stage) { y = parent.height / 2 - height / 2 }
 fun Actor.center(parent: Stage) { centerX(parent); centerY(parent) }
 
 
-fun Actor.surroundWithCircle(size: Float, resizeActor: Boolean = true,
-                             color: Color = Color.WHITE, circleImageLocation:String = "OtherIcons/Circle"): IconCircleGroup {
+fun Actor.surroundWithCircle(
+    size: Float,
+    resizeActor: Boolean = true,
+    color: Color = Color.WHITE,
+    circleImageLocation: String = ImageGetter.circleLocation
+): IconCircleGroup {
     return IconCircleGroup(size, this, resizeActor, color, circleImageLocation)
 }
 
 fun Actor.surroundWithThinCircle(color: Color=Color.BLACK): IconCircleGroup = surroundWithCircle(width+2f, false, color)
 
 
-fun Actor.addBorder(size:Float, color: Color, expandCell:Boolean = false): Table {
+fun Actor.addBorder(size: Float, color: Color, expandCell: Boolean = false): Table {
     val table = Table()
     table.pad(size)
     table.background = BaseScreen.skinStrings.getUiBackground("General/Border", tintColor = color)
@@ -126,9 +139,14 @@ fun Actor.getAscendant(predicate: (Actor) -> Boolean): Actor? {
     return null
 }
 
+/** Gets the nearest parent of this actor that is a [T], or null if none of its parents is of that type. */
+inline fun <reified T> Actor.getAscendant(): T? {
+    return  getAscendant { it is T } as? T
+}
+
 /** The actors bounding box in stage coordinates */
 val Actor.stageBoundingBox: Rectangle get() {
-    val bottomLeft = localToStageCoordinates(Vector2(0f, 0f))
+    val bottomLeft = localToStageCoordinates(Vector2.Zero.cpy())
     val topRight = localToStageCoordinates(Vector2(width, height))
     return Rectangle(
         bottomLeft.x,
@@ -165,7 +183,7 @@ fun Rectangle.getOverlap(other: Rectangle): Rectangle? {
 val Rectangle.top get() = y + height
 val Rectangle.right get() = x + width
 
-fun Group.addBorderAllowOpacity(size:Float, color: Color): Group {
+fun Group.addBorderAllowOpacity(size: Float, color: Color): Group {
     val group = this
     fun getTopBottomBorder() = ImageGetter.getDot(color).apply { width=group.width; height=size }
     addActor(getTopBottomBorder().apply { setPosition(0f, group.height, Align.topLeft) })
@@ -221,6 +239,10 @@ fun Image.setSize(size: Float) {
     setSize(size, size)
 }
 
+/** Proxy for [ScrollPane.scrollTo] using the [bounds][Actor.setBounds] of a given [actor] for its parameters */
+fun ScrollPane.scrollTo(actor: Actor, center: Boolean = false) =
+    scrollTo(actor.x, actor.y, actor.width, actor.height, center, center)
+
 /** Translate a [String] and make a [TextButton] widget from it */
 fun String.toTextButton(style: TextButtonStyle? = null, hideIcons: Boolean = false): TextButton {
     val text = this.tr(hideIcons)
@@ -239,6 +261,24 @@ fun String.toImageButton(iconSize: Float, circleSize: Float, circleColor: Color,
     return button.surroundWithCircle( circleSize, false, circleColor)
 }
 
+/** Return a "close" button, visually a circle with "x" icon that goes red on mouse-over.
+ *
+ *  For use e.g. in the top-right corner of screens such as CivilopediaScreen.
+ *  Automatically binds the BACK key to the [action].
+ */
+fun getCloseButton(
+    size: Float = 50f,
+    iconSize: Float = size - 20f,
+    circleColor: Color = BaseScreen.skinStrings.skinConfig.baseColor,
+    overColor: Color = Color.RED,
+    action: () -> Unit
+): Group {
+    val closeButton = "OtherIcons/Close".toImageButton(iconSize, size, circleColor, overColor)
+    closeButton.onActivation(action)
+    closeButton.keyShortcuts.add(KeyCharAndCode.BACK)
+    return closeButton
+}
+
 /** Translate a [String] and make a [Label] widget from it */
 fun String.toLabel() = Label(this.tr(), BaseScreen.skin)
 /** Make a [Label] widget containing this [Int] as text */
@@ -285,7 +325,7 @@ fun Label.setFontColor(color: Color): Label {
 }
 
 /** Sets the font size on a [Label] and returns it to allow chaining */
-fun Label.setFontSize(size:Int): Label {
+fun Label.setFontSize(size: Int): Label {
     style = Label.LabelStyle(style)
     style.font = Fonts.font
     @Suppress("UsePropertyAccessSyntax") setStyle(style)
@@ -345,8 +385,8 @@ object GdxKeyCodeFixes {
 
     fun toString(keyCode: Int): String = when(keyCode) {
         UNKNOWN -> ""
-        DEL -> "Del"
-        BACKSPACE -> "Backspace"
+        DEL -> "Del"  // Gdx would name this "Forward Delete"
+        BACKSPACE -> "Backspace"  // Gdx would name this "Delete"
         else -> Input.Keys.toString(keyCode)
             ?: ""
     }
@@ -362,5 +402,57 @@ object GdxKeyCodeFixes {
     }
 }
 
+fun Input.isShiftKeyPressed() = isKeyPressed(Input.Keys.SHIFT_LEFT) || isKeyPressed(Input.Keys.SHIFT_RIGHT)
+fun Input.isControlKeyPressed() = isKeyPressed(Input.Keys.CONTROL_LEFT) || isKeyPressed(Input.Keys.CONTROL_RIGHT)
 fun Input.areSecretKeysPressed() = isKeyPressed(Input.Keys.SHIFT_RIGHT) &&
         (isKeyPressed(Input.Keys.CONTROL_RIGHT) || isKeyPressed(Input.Keys.ALT_RIGHT))
+
+/** Sets first row cell's minWidth to the max of the widths of that column over all given tables
+ *
+ * Notes:
+ * - This aligns columns only if the tables are arranged vertically with equal X coordinates.
+ * - first table determines columns processed, all others must have at least the same column count.
+ * - Tables are left as needsLayout==true, so while equal width is ensured, you may have to pack if you want to see the value before this is rendered.
+ * - Note: The receiver <Group> isn't actually needed except to make sure the arguments are descendants.
+ */
+fun equalizeColumns(vararg tables: Table) {
+    for (table in tables) {
+        table.packIfNeeded()
+    }
+    val columns = tables.first().columns
+    check(tables.all { it.columns >= columns }) {
+        "IPageExtensions.equalizeColumns needs all tables to have at least the same number of columns as the first one"
+    }
+
+    val widths = (0 until columns)
+        .mapTo(ArrayList(columns)) { column ->
+            tables.maxOf { it.getColumnWidth(column) }
+        }
+    for (table in tables) {
+        for (column in 0 until columns)
+            table.cells[column].run {
+                if (actor == null)
+                // Empty cells ignore minWidth, so just doing Table.add() for an empty cell in the top row will break this. Fix!
+                    setActor<Label>("".toLabel())
+                else if (Align.isCenterHorizontal(align)) (actor as? Label)?.run {
+                    // minWidth acts like fillX, so Labels will fill and then left-align by default. Fix!
+                    if (!Align.isCenterHorizontal(labelAlign))
+                        setAlignment(Align.center)
+                }
+                minWidth(widths[column] - padLeft - padRight)
+            }
+        table.invalidate()
+    }
+}
+
+/** Retrieve a texture Pixmap without reload or ownership transfer, useable for read operations only.
+ *
+ *  (FileTextureData.consumePixmap forces a reload of the entire file - inefficient if we only want to look at pixel values) */
+fun TextureData.getReadonlyPixmap(): Pixmap {
+    if (!isPrepared) prepare()
+    if (this is PixmapTextureData) return consumePixmap()
+    if (this !is FileTextureData) throw TypeCastException("getReadonlyPixmap only works on file or pixmap based textures")
+    val field = FileTextureData::class.java.getDeclaredField("pixmap")
+    field.isAccessible = true
+    return field.get(this) as Pixmap
+}
