@@ -1,6 +1,5 @@
 package com.unciv.ui.screens.cityscreen
 
-import com.badlogic.gdx.Input
 import com.badlogic.gdx.graphics.Color
 import com.badlogic.gdx.scenes.scene2d.ui.Image
 import com.badlogic.gdx.scenes.scene2d.ui.Table
@@ -9,6 +8,7 @@ import com.unciv.GUI
 import com.unciv.UncivGame
 import com.unciv.logic.automation.Automation
 import com.unciv.logic.city.City
+import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.TutorialTrigger
 import com.unciv.models.UncivSound
@@ -21,12 +21,14 @@ import com.unciv.models.stats.Stat
 import com.unciv.models.translations.tr
 import com.unciv.ui.audio.CityAmbiencePlayer
 import com.unciv.ui.audio.SoundPlayer
+import com.unciv.ui.components.ParticleEffectMapFireworks
 import com.unciv.ui.components.extensions.colorFromRGB
 import com.unciv.ui.components.extensions.disable
 import com.unciv.ui.components.extensions.packIfNeeded
 import com.unciv.ui.components.extensions.toTextButton
 import com.unciv.ui.components.input.KeyCharAndCode
 import com.unciv.ui.components.input.KeyShortcutDispatcherVeto
+import com.unciv.ui.components.input.KeyboardBinding
 import com.unciv.ui.components.input.keyShortcuts
 import com.unciv.ui.components.input.onActivation
 import com.unciv.ui.components.input.onClick
@@ -42,11 +44,16 @@ import com.unciv.ui.popups.closeAllPopups
 import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.ui.screens.basescreen.RecreateOnResize
 import com.unciv.ui.screens.worldscreen.WorldScreen
+import kotlin.math.max
 
 class CityScreen(
     internal val city: City,
     initSelectedConstruction: IConstruction? = null,
-    initSelectedTile: Tile? = null
+    initSelectedTile: Tile? = null,
+    /** City ambience sound player proxies can be passed from one CityScreen instance to the next
+     *  to avoid premature stops or rewinds. Only the fresh CityScreen from WorldScreen or Overview
+     *  will instantiate a new CityAmbiencePlayer and start playing. */
+    ambiencePlayer: CityAmbiencePlayer? = null
 ): BaseScreen(), RecreateOnResize {
     companion object {
         /** Distance from stage edges to floating widgets */
@@ -56,8 +63,19 @@ class CityScreen(
         const val wltkIconSize = 40f
     }
 
+    private val selectedCiv: Civilization = GUI.getWorldScreen().selectedCiv
+
+    private val isSpying = selectedCiv.gameInfo.isEspionageEnabled() && selectedCiv != city.civ
+
+    /**
+     * This is the regular civ city list if we are not spying, if we are spying then it is every foreign city that our spies are in
+     */
+    val viewableCities = if (isSpying) selectedCiv.espionageManager.getCitiesWithOurSpies()
+        .filter { it.civ !=  GUI.getWorldScreen().selectedCiv }
+    else city.civ.cities
+
     /** Toggles or adds/removes all state changing buttons */
-    val canChangeState = GUI.isAllowedChangeState()
+    val canChangeState = GUI.isAllowedChangeState() && !isSpying
 
     // Clockwise from the top-left
 
@@ -119,19 +137,24 @@ class CityScreen(
     // val should be OK as buying tiles is what changes this, and that would re-create the whole CityScreen
     private val nextTileToOwn = city.expansion.chooseNewTileToOwn()
 
-    private val cityAmbiencePlayer = CityAmbiencePlayer(city)
+    private var cityAmbiencePlayer: CityAmbiencePlayer?  = ambiencePlayer ?: CityAmbiencePlayer(city)
+
+    /** Particle effects for WLTK day decoration */
+    private val isWLTKday = city.isWeLoveTheKingDayActive()
+    private val fireworks: ParticleEffectMapFireworks?
 
     init {
-        if (city.isWeLoveTheKingDayActive() && UncivGame.Current.settings.citySoundsVolume > 0) {
+        if (isWLTKday && UncivGame.Current.settings.citySoundsVolume > 0) {
             SoundPlayer.play(UncivSound("WLTK"))
         }
+        fireworks = if (isWLTKday) ParticleEffectMapFireworks.create(game, mapScrollPane) else null
 
         UncivGame.Current.settings.addCompletedTutorialTask("Enter city screen")
 
         addTiles()
 
-        //stage.setDebugTableUnderMouse(true)
         stage.addActor(cityStatsTable)
+        // If we are spying then we shoulden't be able to see their construction screen.
         constructionsTable.addActorsToStage()
         stage.addActor(selectedConstructionTable)
         stage.addActor(tileTable)
@@ -139,17 +162,32 @@ class CityScreen(
         stage.addActor(exitCityButton)
         update()
 
-        globalShortcuts.add(Input.Keys.LEFT) { page(-1) }
-        globalShortcuts.add(Input.Keys.RIGHT) { page(1) }
+        globalShortcuts.add(KeyboardBinding.PreviousCity) { page(-1) }
+        globalShortcuts.add(KeyboardBinding.NextCity) { page(1) }
     }
 
+    override fun getCivilopediaRuleset() = selectedCiv.gameInfo.ruleset
+
     internal fun update() {
         // Recalculate Stats
         city.cityStats.update()
 
-        constructionsTable.isVisible = true
+        constructionsTable.isVisible = !isSpying
         constructionsTable.update(selectedConstruction)
 
+        updateWithoutConstructionAndMap()
+
+        // Rest of screen: Map of surroundings
+        updateTileGroups()
+        if (isPortrait()) mapScrollPane.apply {
+            // center scrolling so city center sits more to the bottom right
+            scrollX = (maxX - constructionsTable.getLowerWidth() - posFromEdge) / 2
+            scrollY = (maxY - cityStatsTable.packIfNeeded().height - posFromEdge + cityPickerTable.top) / 2
+            updateVisualScroll()
+        }
+    }
+
+    internal fun updateWithoutConstructionAndMap() {
         // Bottom right: Tile or selected construction info
         tileTable.update(selectedTile)
         tileTable.setPosition(stage.width - posFromEdge, posFromEdge, Align.bottomRight)
@@ -185,15 +223,6 @@ class CityScreen(
 
         // Top center: Annex/Raze button
         updateAnnexAndRazeCityButton()
-
-        // Rest of screen: Map of surroundings
-        updateTileGroups()
-        if (isPortrait()) mapScrollPane.apply {
-            // center scrolling so city center sits more to the bottom right
-            scrollX = (maxX - constructionsTable.getLowerWidth() - posFromEdge) / 2
-            scrollY = (maxY - cityStatsTable.packIfNeeded().height - posFromEdge + cityPickerTable.top) / 2
-            updateVisualScroll()
-        }
     }
 
     fun canCityBeChanged(): Boolean {
@@ -202,22 +231,19 @@ class CityScreen(
 
     private fun updateTileGroups() {
         val cityUniqueCache = LocalUniqueCache()
-        fun isExistingImprovementValuable(tile: Tile, improvementToPlace: TileImprovement): Boolean {
+        fun isExistingImprovementValuable(tile: Tile): Boolean {
             if (tile.improvement == null) return false
             val civInfo = city.civ
-            val existingStats = tile.stats.getImprovementStats(
+
+            val statDiffForNewImprovement = tile.stats.getStatDiffForImprovement(
                 tile.getTileImprovement()!!,
                 civInfo,
                 city,
                 cityUniqueCache
             )
-            val replacingStats = tile.stats.getImprovementStats(
-                improvementToPlace,
-                civInfo,
-                city,
-                cityUniqueCache
-            )
-            return Automation.rankStatsValue(existingStats, civInfo) > Automation.rankStatsValue(replacingStats, civInfo)
+
+            // If stat diff for new improvement is negative/zero utility, current improvement is valuable
+            return Automation.rankStatsValue(statDiffForNewImprovement, civInfo) <= 0
         }
 
         fun getPickImprovementColor(tile: Tile): Pair<Color, Float> {
@@ -225,12 +251,13 @@ class CityScreen(
             return when {
                 tile.isMarkedForCreatesOneImprovement() -> Color.BROWN to 0.7f
                 !tile.improvementFunctions.canBuildImprovement(improvementToPlace, city.civ) -> Color.RED to 0.4f
-                isExistingImprovementValuable(tile, improvementToPlace) -> Color.ORANGE to 0.5f
+                isExistingImprovementValuable(tile) -> Color.ORANGE to 0.5f
                 tile.improvement != null -> Color.YELLOW to 0.6f
                 tile.turnsToImprovement > 0 -> Color.YELLOW to 0.6f
                 else -> Color.GREEN to 0.5f
             }
         }
+
         for (tileGroup in tileGroups) {
             tileGroup.update()
             tileGroup.layerMisc.removeHexOutline()
@@ -248,6 +275,9 @@ class CityScreen(
                     getPickImprovementColor(tileGroup.tile).run {
                         tileGroup.layerMisc.addHexOutline(first.cpy().apply { this.a = second }) }
             }
+
+            if (fireworks == null || tileGroup.tile.position != city.location) continue
+            fireworks.setActorBounds(tileGroup)
         }
     }
 
@@ -257,7 +287,7 @@ class CityScreen(
         fun addWltkIcon(name: String, apply: Image.()->Unit = {}) =
             razeCityButtonHolder.add(ImageGetter.getImage(name).apply(apply)).size(wltkIconSize)
 
-        if (city.isWeLoveTheKingDayActive()) {
+        if (isWLTKday && fireworks == null) {
             addWltkIcon("OtherIcons/WLTK LR") { color = Color.GOLD }
             addWltkIcon("OtherIcons/WLTK 1") { color = Color.FIREBRICK }.padRight(10f)
         }
@@ -289,7 +319,7 @@ class CityScreen(
             razeCityButtonHolder.add(stopRazingCityButton) //.colspan(cityPickerTable.columns)
         }
 
-        if (city.isWeLoveTheKingDayActive()) {
+        if (isWLTKday && fireworks == null) {
             addWltkIcon("OtherIcons/WLTK 2") { color = Color.FIREBRICK }.padLeft(10f)
             addWltkIcon("OtherIcons/WLTK LR") {
                 color = Color.GOLD
@@ -306,10 +336,11 @@ class CityScreen(
     }
 
     private fun addTiles() {
+        val viewRange = max(city.getExpandRange(), city.getWorkRange())
         val tileSetStrings = TileSetStrings()
-        val cityTileGroups = city.getCenterTile().getTilesInDistance(5)
-                .filter { city.civ.hasExplored(it) }
-                .map { CityTileGroup(city, it, tileSetStrings) }
+        val cityTileGroups = city.getCenterTile().getTilesInDistance(viewRange)
+                .filter { selectedCiv.hasExplored(it) }
+                .map { CityTileGroup(city, it, tileSetStrings, fireworks != null) }
 
         for (tileGroup in cityTileGroups) {
             tileGroup.onClick { tileGroupOnClick(tileGroup, city) }
@@ -322,8 +353,8 @@ class CityScreen(
         for (tileGroup in tileGroups) {
             val xDifference = city.getCenterTile().position.x - tileGroup.tile.position.x
             val yDifference = city.getCenterTile().position.y - tileGroup.tile.position.y
-            //if difference is bigger than 5 the tileGroup we are looking for is on the other side of the map
-            if (xDifference > 5 || xDifference < -5 || yDifference > 5 || yDifference < -5) {
+            //if difference is bigger than the expansion range the tileGroup we are looking for is on the other side of the map
+            if (xDifference > viewRange || xDifference < -viewRange || yDifference > viewRange || yDifference < -viewRange) {
                 //so we want to unwrap its position
                 tilesToUnwrap.add(tileGroup)
             }
@@ -348,7 +379,7 @@ class CityScreen(
         if (!canChangeState || city.isPuppet) return
         val tile = tileGroup.tile
 
-        // Cycling as: Not-worked -> Worked -> Locked -> Not-worked
+        // Cycling as: Not-worked -> Worked  -> Not-worked
         if (tileGroup.tileState == CityTileState.WORKABLE) {
             if (!tile.providesYield() && city.population.getFreePopulation() > 0) {
                 city.workedTiles.add(tile.position)
@@ -399,6 +430,11 @@ class CityScreen(
         if (!canChangeState || city.isPuppet || tileGroup.tileState != CityTileState.WORKABLE) return
         val tile = tileGroup.tile
 
+        // Double-click should lead to locked tiles - both for unworked AND worked tiles
+
+        if (!tile.isWorked()) // If not worked, try to work it first
+            tileWorkedIconOnClick(tileGroup, city)
+
         if (tile.isWorked())
             city.lockedTiles.add(tile.position)
 
@@ -434,13 +470,17 @@ class CityScreen(
         update()
     }
 
+    /** Convenience shortcut to [CivConstructions.hasFreeBuilding][com.unciv.logic.civilization.CivConstructions.hasFreeBuilding], nothing more */
+    internal fun hasFreeBuilding(building: Building) =
+        city.civ.civConstructions.hasFreeBuilding(city, building)
+
     fun selectConstruction(name: String) {
         selectConstruction(city.cityConstructions.getConstruction(name))
     }
     fun selectConstruction(newConstruction: IConstruction) {
         selectedConstruction = newConstruction
         if (newConstruction is Building && newConstruction.hasCreateOneImprovementUnique()) {
-            val improvement = newConstruction.getImprovementToCreate(city.getRuleset())
+            val improvement = newConstruction.getImprovementToCreate(city.getRuleset(), city.civ)
             selectedQueueEntryTargetTile = if (improvement == null) null
                 else city.cityConstructions.getTileForImprovement(improvement.name)
         } else {
@@ -458,7 +498,7 @@ class CityScreen(
     fun clearSelection() = selectTile(null)
 
     fun startPickTileForCreatesOneImprovement(construction: Building, stat: Stat, isBuying: Boolean) {
-        val improvement = construction.getImprovementToCreate(city.getRuleset()) ?: return
+        val improvement = construction.getImprovementToCreate(city.getRuleset(), city.civ) ?: return
         pickTileData = PickTileForImprovementData(construction, improvement, isBuying, stat)
         updateTileGroups()
         ToastPopup("Please select a tile for this building's [${improvement.name}]", this)
@@ -477,21 +517,37 @@ class CityScreen(
         }
     }
 
+    private fun passOnCityAmbiencePlayer(): CityAmbiencePlayer? {
+        val player = cityAmbiencePlayer
+        cityAmbiencePlayer = null
+        return player
+    }
+
     fun page(delta: Int) {
-        val civInfo = city.civ
-        val numCities = civInfo.cities.size
+        // Normal order is create new, then dispose old. But CityAmbiencePlayer delegates to a single instance of MusicController,
+        // leading to one extra play followed by a stop for the city ambience sounds. To avoid that, we pass our player on and relinquish control.
+
+        val numCities = viewableCities.size
         if (numCities == 0) return
-        val indexOfCity = civInfo.cities.indexOf(city)
+        val indexOfCity = viewableCities.indexOf(city)
         val indexOfNextCity = (indexOfCity + delta + numCities) % numCities
-        val newCityScreen = CityScreen(civInfo.cities[indexOfNextCity])
+        val newCityScreen = CityScreen(viewableCities[indexOfNextCity], ambiencePlayer = passOnCityAmbiencePlayer())
         newCityScreen.update()
         game.replaceCurrentScreen(newCityScreen)
     }
 
-    override fun recreate(): BaseScreen = CityScreen(city)
+    // Don't use passOnCityAmbiencePlayer here - continuing play on the replacement screen would be nice,
+    // but the rapid firing of several resize events will get that un-synced, they would no longer stop on leaving.
+    override fun recreate(): BaseScreen = CityScreen(city, selectedConstruction, selectedTile)
 
     override fun dispose() {
-        cityAmbiencePlayer.dispose()
+        cityAmbiencePlayer?.dispose()
+        fireworks?.dispose()
         super.dispose()
     }
+
+    override fun render(delta: Float) {
+        super.render(delta)
+        fireworks?.render(stage, delta)
+    }
 }
