@@ -204,7 +204,7 @@ class CivilopediaScreen(
         fun shouldBeDisplayed(obj: ICivilopediaText) =
             obj is IHasUniques && !obj.isHiddenFromCivilopedia(game.gameInfo, ruleset)
 
-        for (loopCategory in CivilopediaCategories.values()) {
+        for (loopCategory in CivilopediaCategories.entries) {
             if (!religionEnabled && loopCategory == CivilopediaCategories.Belief) continue
             categoryToEntries[loopCategory] =
                 loopCategory.getCategoryIterator(ruleset, tutorialController)
@@ -260,7 +260,7 @@ class CivilopediaScreen(
         entrySelectTable.top()
         entrySelectScroll.setOverscroll(false, false)
         val descriptionTable = Table()
-        descriptionTable.add(flavourTable).row()
+        descriptionTable.add(flavourTable).padTop(7f).padBottom(5f).row()  // 2f of that 7f is used up by Portrait painting e.g. a Nation's outer border *outside its bounds*
         val entrySplitPane = SplitPane(entrySelectScroll, ScrollPane(descriptionTable), false, skin)
         entrySplitPane.splitAmount = 0.3f
         entryTable.addActor(entrySplitPane)
