@@ -2,20 +2,27 @@ package com.unciv.ui.screens.civilopediascreen
 
 import com.badlogic.gdx.Gdx
 import com.badlogic.gdx.graphics.Color
+import com.badlogic.gdx.graphics.Colors
+import com.badlogic.gdx.graphics.Pixmap
+import com.badlogic.gdx.graphics.g2d.TextureRegion
 import com.badlogic.gdx.scenes.scene2d.Actor
+import com.badlogic.gdx.scenes.scene2d.ui.Image
 import com.badlogic.gdx.scenes.scene2d.ui.Table
+import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable
 import com.badlogic.gdx.utils.Align
 import com.unciv.Constants
 import com.unciv.UncivGame
+import com.unciv.logic.UncivShowableException
 import com.unciv.models.metadata.BaseRuleset
 import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.RulesetCache
 import com.unciv.models.ruleset.unique.Unique
-import com.unciv.ui.components.ColorMarkupLabel
+import com.unciv.models.ruleset.validation.RulesetValidator
+import com.unciv.ui.components.extensions.getReadonlyPixmap
 import com.unciv.ui.components.extensions.toLabel
+import com.unciv.ui.components.widgets.ColorMarkupLabel
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.screens.basescreen.BaseScreen
-import com.unciv.ui.components.extensions.toLabel
 import com.unciv.utils.Log
 import kotlin.math.max
 
@@ -61,7 +68,7 @@ class FormattedLine (
     val indent: Int = 0,
     /** Defines vertical padding between rows, defaults to 5f. */
     val padding: Float = Float.NaN,
-    /** Sets text color, accepts 6/3-digit web colors (e.g. #FFA040). */
+    /** Sets text color, accepts 6/3-digit web colors (e.g. #FFA040) or names as defined by Gdx [Colors]. */
     val color: String = "",
     /** Renders a separator line instead of text. Can be combined only with [color] and [size] (line width, default 2) */
     val separator: Boolean = false,
@@ -77,7 +84,7 @@ class FormattedLine (
     // have no backing field, be `by lazy` or use @Transient, Thank you.
 
     /** Looks for linkable ruleset objects in [Unique] parameters and returns a linked [FormattedLine] if successful, a plain one otherwise */
-    constructor(unique: Unique, indent: Int = 0) : this(unique.text, getUniqueLink(unique), indent = indent)
+    constructor(unique: Unique, indent: Int = 0) : this(unique.getDisplayText(), getUniqueLink(unique), indent = indent)
 
     /** Link types that can be used for [FormattedLine.link] */
     enum class LinkType {
@@ -108,26 +115,58 @@ class FormattedLine (
     }
 
     /** Retrieves the parsed [Color] corresponding to the [color] property (String)*/
-    val displayColor: Color by lazy { parseColor() }
+    val displayColor: Color by lazy { parseColor() ?: defaultColor }
 
     /** Returns true if this formatted line will not display anything */
     fun isEmpty(): Boolean = text.isEmpty() && extraImage.isEmpty() &&
             !starred && icon.isEmpty() && link.isEmpty() && !separator
 
-    /** Self-check to potentially support the mod checker
-     * @return `null` if no problems found, or multiline String naming problems.
+    /** Self-check to support the RulesetValidator
+     * @return 0 or more Strings naming problems - all occurrences get the same severity upstream
      */
-    @Suppress("unused")
-    fun unsupportedReason(): String? {
-        val reasons = sequence {
-            if (text.isNotEmpty() && separator) yield("separator and text are incompatible")
-            if (extraImage.isNotEmpty() && link.isNotEmpty()) yield("extraImage and other options except imageSize are incompatible")
-            if (header != 0 && size != Int.MIN_VALUE) yield("use either size or header but not both")
-            // ...
-        }
-        return reasons.joinToString { "\n" }.takeIf { it.isNotEmpty() }
+    fun unsupportedReasons(validator: RulesetValidator) = sequence {
+        if (hasNormalContent() && separator)
+            yield("separator and other options are incompatible")
+        if (link.isNotEmpty() && !(isValidInternalLink(link) || link.hasProtocol()))
+            yield("link is invalid - use internal category/name format or a https:// URL")
+        if (icon.isNotEmpty() && !isValidInternalLink(link))
+            yield("icon is invalid - use internal category/name format")
+        if (header != 0 && size != Int.MIN_VALUE)
+            yield("use either size or header but not both")
+        if (header !in headerSizes.indices)
+            yield("header should be in the range 1..${headerSizes.size - 1}") // Not mentioning 0 is valid too - same as omitting it
+        if (size != Int.MIN_VALUE && size !in 1..100)  // arbitrary
+            yield("size is out of sensible range")
+        if (indent !in 0..100)  // arbitrary
+            yield("indent is out of sensible range")
+        if (color.isNotEmpty())
+            if (parseColor() == null)
+                yield("unknown color \"$color\"")
+            else if (text.isEmpty() && textToDisplay.isEmpty() && !starred && !separator)
+                yield("color set but nothing to apply it to")
+        if (iconCrossed && link.isEmpty() && icon.isEmpty())
+            yield("iconCrossed set without icon or link")
+        if (extraImage.isNotEmpty()) checkExtraImage(validator)
+        if (!imageSize.isNaN() && extraImage.isEmpty())
+            yield("imageSize is only valid for an extraImage")
     }
 
+    private suspend fun SequenceScope<String>.checkExtraImage(validator: RulesetValidator) {
+        if (hasNormalContent() || separator)
+            // not checking centered or padding - these may be implementable
+            yield("extraImage and other options except imageSize are incompatible")
+        // check image exists - but for textures from atlases we can't rely on ImageGetter having cached the appropriate combo???
+        if (ImageGetter.imageExists(extraImage)) return
+        if (ImageGetter.findExternalImage(extraImage) != null) return
+        if (validator.uncachedImageExists(extraImage)) return
+        yield("extraImage not found as either atlas texture or in ExtraImages folder")
+    }
+
+    /** Not one of the exceptions - empty, separator or extraImage. For validation, independent of [isEmpty] which looks for **visible** content */
+    private fun hasNormalContent() =
+        text.isNotEmpty() || link.isNotEmpty() || icon.isNotEmpty() || color.isNotEmpty() || size != Int.MIN_VALUE || header != 0 || starred
+    private fun isValidInternalLink(link: String) = link.matches(Regex("""^[^/]+/[^/]+$"""))
+
     /** Constants used by [FormattedLine] */
     companion object {
         /** Array of text sizes to translate the [header] attribute */
@@ -186,7 +225,7 @@ class FormattedLine (
                 yield(CivilopediaCategories.Wonder to ruleSet.buildings.filter { it.value.isAnyWonder() })
             }
             val result = HashMap<String, CivilopediaCategories>()
-            allObjectMapsSequence.filter { !it.first.hide }
+            allObjectMapsSequence
                 .flatMap { pair -> pair.second.keys.asSequence().map { key -> pair.first to key } }
                 .forEach {
                     result[it.second] = it.first
@@ -215,14 +254,14 @@ class FormattedLine (
         }
 
     /** Parse a json-supplied color string to [Color], defaults to [defaultColor]. */
-    private fun parseColor(): Color {
-        if (color.isEmpty()) return defaultColor
+    private fun parseColor(): Color? {
+        if (color.isEmpty()) return null
         if (color[0] == '#' && color.isHex(1,3)) {
             if (color.isHex(1,6)) return Color.valueOf(color)
             val hex6 = String(charArrayOf(color[1], color[1], color[2], color[2], color[3], color[3]))
             return Color.valueOf(hex6)
         }
-        return defaultColor
+        return Colors.get(color.uppercase())
     }
 
     /** Used only as parameter to [FormattedLine.render] and [MarkupRenderer.render] */
@@ -234,26 +273,7 @@ class FormattedLine (
      * @param iconDisplay Flag to omit link or all images.
      */
     fun render(labelWidth: Float, iconDisplay: IconDisplay = IconDisplay.All): Actor {
-        if (extraImage.isNotEmpty()) {
-            val table = Table(BaseScreen.skin)
-            try {
-                val image = when {
-                    ImageGetter.imageExists(extraImage) ->
-                        ImageGetter.getImage(extraImage)
-                    Gdx.files.internal("ExtraImages/$extraImage.png").exists() ->
-                        ImageGetter.getExternalImage("$extraImage.png")
-                    Gdx.files.internal("ExtraImages/$extraImage.jpg").exists() ->
-                        ImageGetter.getExternalImage("$extraImage.jpg")
-                    else -> return table
-                }
-                val width = if (imageSize.isNaN()) labelWidth else imageSize
-                val height = width * image.height / image.width
-                table.add(image).size(width, height)
-            } catch (exception: Exception) {
-                Log.error("Exception while rendering civilopedia text", exception)
-            }
-            return table
-        }
+        if (extraImage.isNotEmpty()) return renderExtraImage(labelWidth)
 
         val fontSize = when {
             header in 1 until headerSizes.size -> headerSizes[header]
@@ -307,6 +327,30 @@ class FormattedLine (
         return table
     }
 
+    private fun renderExtraImage(labelWidth: Float): Table {
+        val table = Table(BaseScreen.skin)
+        fun getExtraImage(): Image {
+            if (ImageGetter.imageExists(extraImage))
+                return if (centered) ImageGetter.getDrawable(extraImage).cropToContent()
+                    else ImageGetter.getImage(extraImage)
+            val externalImage = ImageGetter.findExternalImage(extraImage)
+                ?: throw UncivShowableException("Extra image '[$extraImage]' not found") // logged in catch below
+            return ImageGetter.getExternalImage(externalImage)
+        }
+        try {
+            val image = getExtraImage()
+            // limit larger cordinate to a given max size
+            val maxSize = if (imageSize.isNaN()) labelWidth else imageSize
+            val (width, height) = if (image.width > image.height)
+                maxSize to maxSize * image.height / image.width
+            else maxSize * image.width / image.height to maxSize
+            table.add(image).size(width, height)
+        } catch (exception: Exception) {
+            Log.error("Exception while rendering civilopedia text", exception)
+        }
+        return table
+    }
+
     /** Place a RuleSet object icon.
      * @return 1 if successful for easy counting
      */
@@ -338,4 +382,86 @@ class FormattedLine (
             else -> "'$text'->$link"
         }
     }
+
+    // region Helpers to crop an image to content
+    private fun TextureRegionDrawable.cropToContent(): Image {
+        val rect = getContentSize()
+        val newRegion = TextureRegion(region.texture, rect.x, rect.y, rect.width, rect.height)
+        return Image(TextureRegionDrawable(newRegion))
+    }
+
+    private fun TextureRegionDrawable.getContentSize(): IntRectangle {
+        val pixMap = region.texture.textureData.getReadonlyPixmap()
+        val result = IntRectangle(region.regionX, region.regionY, region.regionWidth, region.regionHeight) // Not Gdx: integers!
+        val original = result.copy()
+
+        while (result.height > 0 && pixMap.isRowEmpty(result, result.height - 1)) {
+            result.height -= 1
+        }
+        while (result.height > 0 && pixMap.isRowEmpty(result, 0)) {
+            result.y += 1
+            result.height -= 1
+        }
+        while (result.width > 0 && pixMap.isColumnEmpty(result, result.width - 1)) {
+            result.width -= 1
+        }
+        while (result.width > 0 && pixMap.isColumnEmpty(result, 0)) {
+            result.x += 1
+            result.width -= 1
+        }
+
+        result.grow((original.width / 40).coerceAtLeast(1), (original.height / 40).coerceAtLeast(1))
+        return result.intersection(original)
+    }
+
+    private fun Pixmap.isRowEmpty(bounds: IntRectangle, relativeY: Int): Boolean {
+        val y = bounds.y + relativeY
+        return (bounds.x until bounds.x + bounds.width).all {
+            getPixel(it, y) and 255 == 0
+        }
+    }
+
+    private fun Pixmap.isColumnEmpty(bounds: IntRectangle, relativeX: Int): Boolean {
+        val x = bounds.x + relativeX
+        return (bounds.y until bounds.y + bounds.height).all {
+            getPixel(x, it) and 255 == 0
+        }
+    }
+    // endregion
+
+    // region Integer Rectangle class
+    /** Partial rewrite of java.awt.Rectangle which is not available on Android. */
+    private data class IntRectangle(
+        var x: Int,
+        var y: Int,
+        var width: Int,
+        var height: Int
+    ) {
+        // Note: Gdx *has* an Integer equivalent of Vector2: GridPoint2 - but not of Rectangle (all in com.badlogic.gdx.math)
+
+        /** Grow both left and right edges horizontally by [h] and correspondingly top, bottom by [v]
+         *
+         *  Unlike java.awt.Rectangle this will not check for integer overflow or negative size.
+         */
+        fun grow(h: Int, v: Int) {
+            x -= h
+            width += h + h
+            y -= v
+            height += y + y
+        }
+
+        /** Returns a new IntRectangle that represents the intersection of the two rectangles: `this` and [r].
+         *  If the two rectangles do not intersect, the result will be an empty rectangle.
+         *
+         *  Unlike java.awt.Rectangle this will not check for integer overflow or negative size.
+         */
+        fun intersection(r: IntRectangle): IntRectangle {
+            val tx1 = x.coerceAtLeast(r.x)
+            val ty1 = y.coerceAtLeast(r.y)
+            val tx2 = (x + width).coerceAtMost(r.x + r.width)
+            val ty2 = (y + height).coerceAtMost(r.y + r.height)
+            return IntRectangle(tx1, ty1, tx2 - tx1, ty2 - ty1)
+        }
+    }
+    // endregion
 }
