@@ -13,7 +13,7 @@ import com.unciv.logic.map.mapgenerator.MapResourceSetting
 import com.unciv.logic.map.MapShape
 import com.unciv.logic.map.MapSize
 import com.unciv.logic.map.MapType
-import com.unciv.ui.components.UncivTextField
+import com.unciv.ui.components.widgets.UncivTextField
 import com.unciv.ui.components.extensions.pad
 import com.unciv.ui.components.extensions.toCheckBox
 import com.unciv.ui.components.extensions.toLabel
@@ -121,7 +121,7 @@ class MapParametersTable(
             add(optionsTable).colspan(2).grow().row()
         } else {
             val mapShapeSelectBox =
-                    TranslatedSelectBox(mapShapes, mapParameters.shape, skin)
+                    TranslatedSelectBox(mapShapes, mapParameters.shape)
             mapShapeSelectBox.onChange {
                 mapParameters.shape = mapShapeSelectBox.selected.value
                 updateWorldSizeTable()
@@ -160,7 +160,7 @@ class MapParametersTable(
             }
             add(optionsTable).colspan(2).grow().row()
         } else {
-            mapTypeSelectBox = TranslatedSelectBox(mapTypes, mapParameters.type, skin)
+            mapTypeSelectBox = TranslatedSelectBox(mapTypes, mapParameters.type)
 
             mapTypeSelectBox.onChange {
                 mapParameters.type = mapTypeSelectBox.selected.value
@@ -189,7 +189,7 @@ class MapParametersTable(
             add(optionsTable).colspan(2).grow().row()
         } else {
             val mapSizes = MapSize.names() + listOf(MapSize.custom)
-            worldSizeSelectBox = TranslatedSelectBox(mapSizes, mapParameters.mapSize.name, skin)
+            worldSizeSelectBox = TranslatedSelectBox(mapSizes, mapParameters.mapSize.name)
             worldSizeSelectBox.onChange { updateWorldSizeTable() }
 
             addHexagonalSizeTable()
@@ -205,7 +205,7 @@ class MapParametersTable(
 
     private fun addHexagonalSizeTable() {
         val defaultRadius = mapParameters.mapSize.radius.toString()
-        customMapSizeRadius = UncivTextField.create("Radius", defaultRadius).apply {
+        customMapSizeRadius = UncivTextField("Radius", defaultRadius).apply {
             textFieldFilter = DigitsOnlyFilter()
         }
         customMapSizeRadius.onChange {
@@ -219,12 +219,12 @@ class MapParametersTable(
 
     private fun addRectangularSizeTable() {
         val defaultWidth = mapParameters.mapSize.width.toString()
-        customMapWidth = UncivTextField.create("Width", defaultWidth).apply {
+        customMapWidth = UncivTextField("Width", defaultWidth).apply {
             textFieldFilter = DigitsOnlyFilter()
         }
 
         val defaultHeight = mapParameters.mapSize.height.toString()
-        customMapHeight = UncivTextField.create("Height", defaultHeight).apply {
+        customMapHeight = UncivTextField("Height", defaultHeight).apply {
             textFieldFilter = DigitsOnlyFilter()
         }
 
@@ -271,7 +271,7 @@ class MapParametersTable(
             }
             add(optionsTable).colspan(2).grow().row()
         } else {
-            resourceSelectBox = TranslatedSelectBox(mapResources, mapParameters.mapResources, skin)
+            resourceSelectBox = TranslatedSelectBox(mapResources, mapParameters.mapResources)
 
             resourceSelectBox.onChange {
                 mapParameters.mapResources = resourceSelectBox.selected.value
@@ -360,7 +360,7 @@ class MapParametersTable(
     private fun addAdvancedControls(table: Table) {
         table.defaults().pad(5f)
 
-        seedTextField = UncivTextField.create("RNG Seed", mapParameters.seed.toString())
+        seedTextField = UncivTextField("RNG Seed", mapParameters.seed.toString())
         seedTextField.textFieldFilter = DigitsOnlyFilter()
 
         // If the field is empty, fallback seed value to 0
