@@ -1,8 +1,10 @@
 package com.unciv.ui.screens.overviewscreen
 
 import com.badlogic.gdx.graphics.Color
+import com.badlogic.gdx.math.Vector2
 import com.badlogic.gdx.scenes.scene2d.Actor
 import com.badlogic.gdx.scenes.scene2d.Group
+import com.badlogic.gdx.scenes.scene2d.Stage
 import com.badlogic.gdx.scenes.scene2d.Touchable
 import com.badlogic.gdx.scenes.scene2d.ui.Table
 import com.badlogic.gdx.scenes.scene2d.ui.TextButton
@@ -15,24 +17,30 @@ import com.unciv.logic.civilization.diplomacy.DiplomaticStatus
 import com.unciv.logic.civilization.diplomacy.RelationshipLevel
 import com.unciv.logic.map.HexMath
 import com.unciv.models.ruleset.Policy.PolicyBranchType
+import com.unciv.models.ruleset.nation.getContrastRatio
 import com.unciv.models.ruleset.unique.UniqueType
-import com.unciv.ui.components.AutoScrollPane
-import com.unciv.ui.components.ColorMarkupLabel
-import com.unciv.ui.components.Fonts
 import com.unciv.ui.components.UncivTooltip.Companion.addTooltip
 import com.unciv.ui.components.extensions.addBorder
 import com.unciv.ui.components.extensions.addSeparator
 import com.unciv.ui.components.extensions.addSeparatorVertical
 import com.unciv.ui.components.extensions.center
+import com.unciv.ui.components.extensions.equalizeColumns
 import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.components.extensions.toTextButton
+import com.unciv.ui.components.fonts.Fonts
+import com.unciv.ui.components.input.ClickableCircle
+import com.unciv.ui.components.input.onActivation
 import com.unciv.ui.components.input.onClick
+import com.unciv.ui.components.widgets.AutoScrollPane
+import com.unciv.ui.components.widgets.ColorMarkupLabel
+import com.unciv.ui.components.widgets.ShadowedLabel
 import com.unciv.ui.images.ImageGetter
+import com.unciv.ui.popups.AnimatedMenuPopup
 import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.ui.screens.diplomacyscreen.DiplomacyScreen
 import kotlin.math.roundToInt
 
-class GlobalPoliticsOverviewTable (
+class GlobalPoliticsOverviewTable(
     viewingPlayer: Civilization,
     overviewScreen: EmpireOverviewScreen,
     persistedData: EmpireOverviewTabPersistableData? = null
@@ -66,17 +74,31 @@ class GlobalPoliticsOverviewTable (
 
 
     init {
+        top()
         if (persistableData.showDiagram) updateDiagram()
         else updatePoliticsTable()
     }
 
+    override fun getFixedContent() = fixedContent
+
+    //region Politics Table View
+
     private fun updatePoliticsTable() {
         persistableData.showDiagram = false
-        clear()
-        fixedContent.clear()
+        createGlobalPoliticsHeader()
+        createGlobalPoliticsTable()
+        equalizeColumns(fixedContent, this)
+    }
 
+    /** Clears fixedContent and adds the header cells.
+     *  Needs to stay matched to [createGlobalPoliticsTable].
+     *
+     *  9 Columns: 5 info, 4 separators. The first column gets an empty header, the content below is the civ image
+     */
+    private fun createGlobalPoliticsHeader() = fixedContent.run {
         val diagramButton = "Show diagram".toTextButton().onClick(::updateDiagram)
 
+        clear()
         add()
         addSeparatorVertical(Color.GRAY)
         add("Civilization Info".toLabel())
@@ -86,16 +108,22 @@ class GlobalPoliticsOverviewTable (
         add("Wonders".toLabel())
         addSeparatorVertical(Color.GRAY)
         add(Table().apply {
-            add("Relations".toLabel()).row()
+            add("Relations".toLabel()).padTop(10f).row()
             add(diagramButton).pad(10f)
         })
-
-        createGlobalPoliticsTable()
+        addSeparator(Color.GRAY)
     }
 
+    /** Clears [EmpireOverviewTab]'s main Table and adds data columns/rows.
+     *  Needs to stay matched to [createGlobalPoliticsHeader].
+     */
     private fun createGlobalPoliticsTable() {
+        clear()
+
         for (civ in sequenceOf(viewingPlayer) + viewingPlayer.diplomacyFunctions.getKnownCivsSorted(includeCityStates = false)) {
-            addSeparator(Color.GRAY)
+            // We already have a separator under the fixed header, we only need them here between rows.
+            // This is also the replacement for calling row() explicitly
+            if (cells.size > 0) addSeparator(Color.GRAY)
 
             // civ image
             add(ImageGetter.getNationPortrait(civ.nation, 100f)).pad(20f)
@@ -178,6 +206,11 @@ class GlobalPoliticsOverviewTable (
         if (!viewingPlayer.knows(civ) && civ.civName != viewingPlayer.civName)
             return politicsTable
 
+        if (civ.isDefeated()) {
+            politicsTable.add("{Defeated} ${Fonts.death}".toLabel())
+            return politicsTable
+        }
+
         // wars
         for (otherCiv in civ.getKnownCivs()) {
             if (civ.isAtWarWith(otherCiv)) {
@@ -187,9 +220,14 @@ class GlobalPoliticsOverviewTable (
         }
         politicsTable.row()
 
-        // declaration of friendships
+        // defensive pacts and declaration of friendships
         for (otherCiv in civ.getKnownCivs()) {
-            if (civ.diplomacy[otherCiv.civName]?.hasFlag(DiplomacyFlags.DeclarationOfFriendship) == true) {
+            if (civ.diplomacy[otherCiv.civName]?.hasFlag(DiplomacyFlags.DefensivePact) == true) {
+                val friendText = ColorMarkupLabel("Defensive pact with [${getCivName(otherCiv)}]", Color.CYAN)
+                val turnsLeftText = " (${civ.diplomacy[otherCiv.civName]?.getFlag(DiplomacyFlags.DefensivePact)} ${Fonts.turn})".toLabel()
+                politicsTable.add(friendText)
+                politicsTable.add(turnsLeftText).row()
+            } else if (civ.diplomacy[otherCiv.civName]?.hasFlag(DiplomacyFlags.DeclarationOfFriendship) == true) {
                 val friendText = ColorMarkupLabel("Friends with [${getCivName(otherCiv)}]", Color.GREEN)
                 val turnsLeftText = " (${civ.diplomacy[otherCiv.civName]?.getFlag(DiplomacyFlags.DeclarationOfFriendship)} ${Fonts.turn})".toLabel()
                 politicsTable.add(friendText)
@@ -212,7 +250,7 @@ class GlobalPoliticsOverviewTable (
         //allied CS
         for (cityState in gameInfo.getAliveCityStates()) {
             if (cityState.diplomacy[civ.civName]?.isRelationshipLevelEQ(RelationshipLevel.Ally) == true) {
-                val alliedText = ColorMarkupLabel("Allied with [${getCivName(cityState)}]", Color.GREEN)
+                val alliedText = ColorMarkupLabel("Allied with [${getCivName(cityState)}]", Color.CYAN)
                 politicsTable.add(alliedText).row()
             }
         }
@@ -220,12 +258,16 @@ class GlobalPoliticsOverviewTable (
         return politicsTable
     }
 
-    override fun getFixedContent() = fixedContent
+    //endregion
+    //region Diagram View ("Ball of Yarn")
 
     // Refresh content and determine landscape/portrait layout
     private fun updateDiagram() {
         persistableData.showDiagram = true
-        val politicsButton = "Show global politics".toTextButton().onClick(::updatePoliticsTable)
+        val politicsButton = "Show global politics".toTextButton().onClick {
+            updatePoliticsTable()
+            overviewScreen.resizePage(this)  // Or else the header stays curernt size, which with any non-empty diagram is most of the client area
+        }
 
         val toggleCityStatesButton: TextButton = Constants.cityStates.toTextButton().apply {
             onClick {
@@ -247,7 +289,7 @@ class GlobalPoliticsOverviewTable (
             persistableData.includeCityStates && viewingPlayer.hideCityStateCount()
         relevantCivsCount = if (hideCivsCount) "?"
             else gameInfo.civilizations.count {
-                !it.isSpectator() && !it.isBarbarian() && (persistableData.includeCityStates || !it.isCityState())
+                !it.isSpectator() && !it.isBarbarian && (persistableData.includeCityStates || !it.isCityState)
             }.toString()
         undefeatedCivs = sequenceOf(viewingPlayer) +
                 viewingPlayer.diplomacyFunctions.getKnownCivsSorted(persistableData.includeCityStates)
@@ -293,7 +335,7 @@ class GlobalPoliticsOverviewTable (
     /** Same as [Civilization.hideCivCount] but for City-States instead of Major Civs */
     private fun Civilization.hideCityStateCount(): Boolean {
         if (!gameInfo.gameParameters.randomNumberOfCityStates) return false
-        val knownCivs = 1 + getKnownCivs().count { it.isCityState() }
+        val knownCivs = 1 + getKnownCivs().count { it.isCityState }
         if (knownCivs >= gameInfo.gameParameters.maxNumberOfCityStates) return false
         if (hasUnique(UniqueType.OneTimeRevealEntireMap)) return false
         return true
@@ -323,7 +365,9 @@ class GlobalPoliticsOverviewTable (
         add("[$relevantCivsCount] Civilizations in the game".toLabel()).colspan(columns).row()
         add("Our Civilization:".toLabel()).colspan(columns).left().padLeft(10f).padTop(10f).row()
         add(getCivMiniTable(viewingPlayer)).left()
-        add(viewingPlayer.calculateTotalScore().toInt().toLabel()).left().row()
+        val scoreText = if (viewingPlayer.isDefeated()) Fonts.death.toString()
+            else viewingPlayer.calculateTotalScore().toInt().toString()
+        add(scoreText.toLabel()).left().row()
         val turnsTillNextDiplomaticVote = viewingPlayer.getTurnsTillNextDiplomaticVote() ?: return
         add("Turns until the next\ndiplomacy victory vote: [$turnsTillNextDiplomaticVote]".toLabel()).colspan(columns).row()
     }
@@ -348,10 +392,10 @@ class GlobalPoliticsOverviewTable (
         }
 
         for (civ in civs) {
-            if (lastCivWasMajor && civ.isCityState())
+            if (lastCivWasMajor && civ.isCityState)
                 advanceCols(columns)
             add(getCivMiniTable(civ)).left()
-            if (civ.isCityState()) {
+            if (civ.isCityState) {
                 advanceCols(1)
             } else {
                 add(civ.calculateTotalScore().toInt().toLabel()).left()
@@ -360,7 +404,10 @@ class GlobalPoliticsOverviewTable (
         }
     }
 
-    /** This is the 'spider net'-like polygon showing one line per civ-civ relation */
+    /** This is the 'spider net'-like polygon showing one line per civ-civ relation
+     *  @param undefeatedCivs Civs to display - note the viewing player is always included, so it's possible the name is off and there's a dead civ included.
+     *  @param freeSize Width and height this [Group] sizes itself to
+     */
     private class DiplomacyGroup(
         undefeatedCivs: Sequence<Civilization>,
         freeSize: Float
@@ -401,6 +448,16 @@ class GlobalPoliticsOverviewTable (
 
         init {
             setSize(freeSize, freeSize)
+
+            // An Image Actor does not respect alpha for its hit area, it's always square, but we want a clickable _circle_
+            // Radius to show legend should be no larger than freeSize / 2.25f - 15f (see below), let's make it a little smaller
+            val clickableArea = ClickableCircle(freeSize / 1.25f - 25f)
+            clickableArea.onActivation {
+                DiagramLegendPopup(stage, this)
+            }
+            clickableArea.center(this)
+            addActor(clickableArea)
+
             val civGroups = HashMap<String, Actor>()
             val civLines = HashMap<String, MutableSet<Actor>>()
             val civCount = undefeatedCivs.count()
@@ -421,9 +478,11 @@ class GlobalPoliticsOverviewTable (
                 addActor(civGroup)
             }
 
-            for (civ in undefeatedCivs)
+            for (civ in undefeatedCivs) {
+                if (civ.isDefeated()) continue // if you're dead, icon but no lines (One more turn mode after losing)
                 for (diplomacy in civ.diplomacy.values) {
-                    if (diplomacy.otherCiv() !in undefeatedCivs) continue
+                    val otherCiv = diplomacy.otherCiv()
+                    if (otherCiv !in undefeatedCivs || otherCiv.isDefeated()) continue
                     val civGroup = civGroups[civ.civName]!!
                     val otherCivGroup = civGroups[diplomacy.otherCivName]!!
 
@@ -432,9 +491,14 @@ class GlobalPoliticsOverviewTable (
                         startY = civGroup.y + civGroup.height / 2,
                         endX = otherCivGroup.x + otherCivGroup.width / 2,
                         endY = otherCivGroup.y + otherCivGroup.height / 2,
-                        width = 2f)
+                        width = 2f
+                    )
 
                     statusLine.color = if (diplomacy.diplomaticStatus == DiplomaticStatus.War) Color.RED
+                    else if (diplomacy.diplomaticStatus == DiplomaticStatus.DefensivePact
+                        || (diplomacy.civInfo.isCityState && diplomacy.civInfo.getAllyCiv() == diplomacy.otherCivName)
+                        || (otherCiv.isCityState && otherCiv.getAllyCiv() == diplomacy.civInfo.civName)
+                    ) Color.CYAN
                     else diplomacy.relationshipLevel().color
 
                     if (!civLines.containsKey(civ.civName)) civLines[civ.civName] = mutableSetOf()
@@ -442,6 +506,51 @@ class GlobalPoliticsOverviewTable (
 
                     addActorAt(0, statusLine)
                 }
+            }
+
         }
     }
+
+    private class DiagramLegendPopup(stage: Stage, diagram: Actor) : AnimatedMenuPopup(stage, diagram.getCenterInStageCoordinates()) {
+        init {
+            touchable = Touchable.enabled
+            onActivation { close() }
+        }
+
+        companion object {
+            private fun Actor.getCenterInStageCoordinates(): Vector2 = localToStageCoordinates(Vector2(width / 2, height / 2))
+
+            const val lineWidth = 3f  // a little thicker than the actual diagram
+            const val lowContrastWidth = 4f
+            const val lineLength = 120f
+        }
+
+        override fun createContentTable(): Table {
+            val legend = Table()
+            legend.background = ImageGetter.getDrawable("OtherIcons/Politics-diagram-bg")
+            legend.add(ShadowedLabel("Diagram line colors", Constants.headingFontSize)).colspan(2).row()
+            //todo Rethink hardcoding together with the statusLine.color one in DiplomacyGroup
+            legend.addLegendRow("War", Color.RED)
+            for (level in RelationshipLevel.values()) {
+                val lineColor = if (level == RelationshipLevel.Ally) Color.CYAN else level.color
+                legend.addLegendRow(level.name, lineColor)
+            }
+            legend.addLegendRow(Constants.defensivePact, Color.CYAN)
+            return super.createContentTable()!!.apply {
+                add(legend).grow()
+            }
+        }
+
+        fun Table.addLegendRow(text: String, color: Color) {
+            // empiric hack to equalize the "visual impact" a little. Afraid is worst at contrast 1.4, Enemy has 9.8
+            val contrast = getContrastRatio(Color.DARK_GRAY, color).toFloat()
+            val width = lineWidth + (lowContrastWidth - lineWidth) / contrast.coerceAtLeast(1f)
+            val line = ImageGetter.getLine(0f, width / 2, lineLength, width / 2, width)
+            line.color = color
+            add(line).size(lineLength, width).padTop(5f)
+            add(ShadowedLabel(text)).padLeft(5f).padTop(10f).row()
+        }
+    }
+
+    //endregion
 }
