@@ -6,9 +6,9 @@ import com.badlogic.gdx.scenes.scene2d.ui.Table
 import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.civilization.Notification
 import com.unciv.logic.civilization.NotificationCategory
-import com.unciv.ui.components.ColorMarkupLabel
-import com.unciv.ui.components.TabbedPager
-import com.unciv.ui.components.extensions.onClick
+import com.unciv.ui.components.widgets.ColorMarkupLabel
+import com.unciv.ui.components.widgets.TabbedPager
+import com.unciv.ui.components.input.onClick
 import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.screens.basescreen.BaseScreen
@@ -19,10 +19,8 @@ class NotificationsOverviewTable(
     persistedData: EmpireOverviewTabPersistableData? = null
 ) : EmpireOverviewTab(viewingPlayer, overviewScreen) {
     class NotificationsTabPersistableData(
-            var scrollY: Float? = null
-    ) : EmpireOverviewTabPersistableData() {
-        override fun isEmpty() = scrollY == null
-    }
+        var scrollY: Float? = null
+    ) : EmpireOverviewTabPersistableData()
     override val persistableData = (persistedData as? NotificationsTabPersistableData) ?: NotificationsTabPersistableData()
     override fun activated(index: Int, caption: String, pager: TabbedPager) {
         if (persistableData.scrollY != null)
@@ -72,8 +70,8 @@ class NotificationsOverviewTable(
             add(ImageGetter.getWhiteDot()).minHeight(2f).width(stageWidth / 4)
         }).row()
 
-        for (category in NotificationCategory.values()){
-            val categoryNotifications = notifications.filter { it.category == category.name }
+        for (category in NotificationCategory.values()) {
+            val categoryNotifications = notifications.filter { it.category == category }
             if (categoryNotifications.isEmpty()) continue
 
             if (category != NotificationCategory.General)
@@ -88,8 +86,8 @@ class NotificationsOverviewTable(
                 notificationTable.add(label).width(stageWidth / 2 - iconSize * notification.icons.size)
                 notificationTable.background = BaseScreen.skinStrings.getUiBackground("OverviewScreen/NotificationOverviewTable/Notification", BaseScreen.skinStrings.roundedEdgeRectangleShape)
                 notificationTable.touchable = Touchable.enabled
-                if (notification.action != null)
-                    notificationTable.onClick { showOneTimeNotification(notification) }
+                if (notification.actions.isNotEmpty())
+                    notificationTable.onClick { overviewScreen.showOneTimeNotification(notification) }
 
                 notification.addNotificationIconsTo(notificationTable, gameInfo.ruleset, iconSize)
 
