@@ -1,48 +1,46 @@
 package com.unciv.ui.screens.worldscreen.unit.actions
 
 import com.unciv.GUI
+import com.unciv.UncivGame
 import com.unciv.logic.civilization.NotificationCategory
 import com.unciv.logic.civilization.NotificationIcon
 import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.UnitAction
 import com.unciv.models.UnitActionType
+import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.stats.Stat
 import com.unciv.models.stats.Stats
 import com.unciv.ui.popups.ConfirmPopup
-import com.unciv.ui.popups.hasOpenPopups
 import kotlin.random.Random
 
 object UnitActionsPillage {
 
-    fun addPillageAction(unit: MapUnit, actionList: ArrayList<UnitAction>) {
-        val pillageAction = getPillageAction(unit)
-            ?: return
-        if (pillageAction.action == null)
-            actionList += pillageAction
-        else actionList += UnitAction(UnitActionType.Pillage, pillageAction.title) {
-            if (!GUI.getWorldScreen().hasOpenPopups()) {
-                val pillageText = "Are you sure you want to pillage this [${unit.currentTile.getImprovementToPillageName()!!}]?"
-                ConfirmPopup(
-                    GUI.getWorldScreen(),
-                    pillageText,
-                    "Pillage",
-                    true
-                ) {
-                    (pillageAction.action)()
-                    GUI.setUpdateWorldOnNextRender()
-                }.open()
-            }
-        }
+    internal fun getPillageActions(unit: MapUnit, tile: Tile): Sequence<UnitAction> {
+        val pillageAction = getPillageAction(unit, tile)
+            ?: return emptySequence()
+        if (pillageAction.action == null || unit.civ.isAIOrAutoPlaying())
+            return sequenceOf(pillageAction)
+        else return sequenceOf(UnitAction(UnitActionType.Pillage, 65f, pillageAction.title) {
+            val pillageText = "Are you sure you want to pillage this [${tile.getImprovementToPillageName()!!}]?"
+            ConfirmPopup(
+                GUI.getWorldScreen(),
+                pillageText,
+                "Pillage",
+                true
+            ) {
+                (pillageAction.action)()
+                GUI.setUpdateWorldOnNextRender()
+            }.open()
+        })
     }
 
-    fun getPillageAction(unit: MapUnit): UnitAction? {
-        val tile = unit.currentTile
+    internal fun getPillageAction(unit: MapUnit, tile: Tile): UnitAction? {
         val improvementName = unit.currentTile.getImprovementToPillageName()
         if (unit.isCivilian() || improvementName == null || tile.getOwner() == unit.civ) return null
         return UnitAction(
-            UnitActionType.Pillage,
+            UnitActionType.Pillage, 65f,
             title = "${UnitActionType.Pillage} [$improvementName]",
             action = {
                 val pillagedImprovement = unit.currentTile.getImprovementToPillageName()!!  // can this differ from improvementName due to later execution???
@@ -67,7 +65,7 @@ object UnitActionsPillage {
 
                 if (pillagingImprovement)  // only Improvements heal HP
                     unit.healBy(25)
-            }.takeIf { unit.currentMovement > 0 && UnitActions.canPillage(unit, tile) }
+            }.takeIf { unit.currentMovement > 0 && canPillage(unit, tile) }
         )
     }
 
@@ -77,14 +75,15 @@ object UnitActionsPillage {
 
         // Accumulate the loot
         val pillageYield = Stats()
-        for (unique in improvement.getMatchingUniques(UniqueType.PillageYieldRandom)) {
+        val stateForConditionals = StateForConditionals(unit=unit)
+        for (unique in improvement.getMatchingUniques(UniqueType.PillageYieldRandom, stateForConditionals)) {
             for ((stat, value) in unique.stats) {
                 // Unique text says "approximately [X]", so we add 0..X twice - think an RPG's 2d12
                 val looted = Random.nextInt((value + 1).toInt()) + Random.nextInt((value + 1).toInt())
                 pillageYield.add(stat, looted.toFloat())
             }
         }
-        for (unique in improvement.getMatchingUniques(UniqueType.PillageYieldFixed)) {
+        for (unique in improvement.getMatchingUniques(UniqueType.PillageYieldFixed, stateForConditionals)) {
             pillageYield.add(unique.stats)
         }
 
@@ -114,4 +113,14 @@ object UnitActionsPillage {
         toCityPillageYield.notify(" which has been sent to [${closestCity?.name}]")
         globalPillageYield.notify("")
     }
+
+    // Public - used in UnitAutomation
+    fun canPillage(unit: MapUnit, tile: Tile): Boolean {
+        if (unit.isTransported) return false
+        if (!tile.canPillageTile()) return false
+        if (unit.hasUnique(UniqueType.CannotPillage)) return false
+        val tileOwner = tile.getOwner()
+        // Can't pillage friendly tiles, just like you can't attack them - it's an 'act of war' thing
+        return tileOwner == null || unit.civ.isAtWarWith(tileOwner)
+    }
 }
