@@ -9,7 +9,7 @@ import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.stats.Stats
 import com.unciv.testing.GdxTestRunner
-import com.unciv.uniques.TestGame
+import com.unciv.testing.TestGame
 import org.junit.Assert
 import org.junit.Before
 import org.junit.Test
@@ -30,12 +30,11 @@ class TileImprovementConstructionTests {
         testGame.makeHexagonalMap(4)
         tileMap = testGame.tileMap
         civInfo = testGame.addCiv()
-        civInfo.tech.researchedTechnologies.addAll(testGame.ruleset.technologies.values)
-        civInfo.tech.techsResearched.addAll(testGame.ruleset.technologies.keys)
+        for (tech in testGame.ruleset.technologies.values)
+            civInfo.tech.addTechnology(tech.name)
         city = testGame.addCity(civInfo, tileMap[0,0])
     }
 
-
     @Test
     fun allTerrainSpecificImprovementsCanBeBuilt() {
         for (improvement in testGame.ruleset.tileImprovements.values) {
@@ -56,8 +55,8 @@ class TileImprovementConstructionTests {
             }
             tile.setTransients()
 
-            if (improvement.uniqueTo != null){
-                civInfo.civName = improvement.uniqueTo!!
+            if (improvement.uniqueTo != null) {
+                civInfo.setNameForUnitTests(improvement.uniqueTo!!)
             }
 
             val canBeBuilt = tile.improvementFunctions.canBuildImprovement(improvement, civInfo)
@@ -93,7 +92,7 @@ class TileImprovementConstructionTests {
 
         for (improvement in testGame.ruleset.tileImprovements.values) {
             if (!improvement.uniques.contains("Can only be built on [Coastal] tiles")) continue
-            civInfo.civName = improvement.uniqueTo ?: "OtherCiv"
+            civInfo.setNameForUnitTests(improvement.uniqueTo ?: "OtherCiv")
             val canBeBuilt = coastalTile.improvementFunctions.canBuildImprovement(improvement, civInfo)
             Assert.assertTrue(improvement.name, canBeBuilt)
         }
@@ -105,7 +104,7 @@ class TileImprovementConstructionTests {
 
         for (improvement in testGame.ruleset.tileImprovements.values) {
             if (!improvement.uniques.contains("Can only be built on [Coastal] tiles")) continue
-            civInfo.civName = improvement.uniqueTo ?: "OtherCiv"
+            civInfo.setNameForUnitTests(improvement.uniqueTo ?: "OtherCiv")
             val canBeBuilt = tile.improvementFunctions.canBuildImprovement(improvement, civInfo)
             Assert.assertFalse(improvement.name, canBeBuilt)
         }
@@ -115,7 +114,7 @@ class TileImprovementConstructionTests {
     fun uniqueToOtherImprovementsCanNOTBeBuilt() {
         for (improvement in testGame.ruleset.tileImprovements.values) {
             if (improvement.uniqueTo == null) continue
-            civInfo.civName = "OtherCiv"
+            civInfo.setNameForUnitTests("OtherCiv")
             val tile = tileMap[1,1]
             val canBeBuilt = tile.improvementFunctions.canBuildImprovement(improvement, civInfo)
             Assert.assertFalse(improvement.name, canBeBuilt)
@@ -148,7 +147,7 @@ class TileImprovementConstructionTests {
         tile.addTerrainFeature("Forest")
         Assert.assertEquals(tile.terrainFeatures, listOf("Hill", "Forest"))
 
-        tile.changeImprovement("Landmark")
+        tile.setImprovement("Landmark")
         Assert.assertEquals(tile.terrainFeatures, listOf("Hill"))
     }
 
@@ -156,7 +155,7 @@ class TileImprovementConstructionTests {
     fun citadelTakesOverAdjacentTiles() {
         val tile = tileMap[1,1]
         Assert.assertFalse(tile.neighbors.all { it.owningCity == city })
-        tile.changeImprovement("Citadel", civInfo)
+        tile.setImprovement("Citadel", civInfo)
         Assert.assertTrue(tile.neighbors.all { it.owningCity == city })
     }
 
@@ -166,7 +165,7 @@ class TileImprovementConstructionTests {
         tile.resource = "Sheep"
         tile.setTransients()
         tile.addTerrainFeature("Hill")
-        civInfo.civName = "Inca"
+        civInfo.setNameForUnitTests("Inca")
 
         for (improvement in testGame.ruleset.tileImprovements.values) {
             if (!improvement.uniques.contains("Cannot be built on [Bonus resource] tiles")) continue
@@ -176,52 +175,85 @@ class TileImprovementConstructionTests {
     }
 
     @Test
-    fun buildingRoadBuildsARoad(){
+    fun buildingRoadBuildsARoad() {
         val tile = tileMap[1,1]
-        tile.improvementFunctions.changeImprovement("Road")
+        tile.improvementFunctions.setImprovement("Road")
         assert(tile.roadStatus == RoadStatus.Road)
     }
 
     @Test
-    fun removingRoadRemovesRoad(){
+    fun removingRoadRemovesRoad() {
         val tile = tileMap[1,1]
         tile.roadStatus = RoadStatus.Road
-        tile.improvementFunctions.changeImprovement("Remove Road")
+        tile.improvementFunctions.setImprovement("Remove Road")
         assert(tile.roadStatus == RoadStatus.None)
     }
 
     @Test
-    fun removingForestRemovesForestAndLumbermill(){
+    fun removingForestRemovesForestAndLumbermill() {
         val tile = tileMap[1,1]
         tile.addTerrainFeature("Forest")
-        tile.improvementFunctions.changeImprovement("Lumber mill")
+        tile.improvementFunctions.setImprovement("Lumber mill")
         assert(tile.getTileImprovement()!!.name == "Lumber mill")
-        tile.improvementFunctions.changeImprovement("Remove Forest")
+        tile.improvementFunctions.setImprovement("Remove Forest")
         assert(tile.terrainFeatures.isEmpty())
         assert(tile.improvement == null) // Lumber mill can ONLY be on Forest, and is therefore removed
     }
 
     @Test
-    fun removingForestRemovesForestButNotCamp(){
+    fun removingForestRemovesForestButNotCamp() {
         val tile = tileMap[1,1]
         tile.addTerrainFeature("Forest")
         tile.resource = "Deer"
         tile.baseTerrain = "Plains"
-        tile.improvementFunctions.changeImprovement("Camp")
+        tile.improvementFunctions.setImprovement("Camp")
         assert(tile.getTileImprovement()!!.name == "Camp")
-        tile.improvementFunctions.changeImprovement("Remove Forest")
+        tile.improvementFunctions.setImprovement("Remove Forest")
         assert(tile.terrainFeatures.isEmpty())
         assert(tile.improvement == "Camp") // Camp can be both on Forest AND on Plains, so not removed
     }
 
     @Test
-    fun statsDiffFromRemovingForestTakesRemovedLumberMillIntoAccount(){
+    fun improvementCannotBuildWhenNotAllowed() {
+        val tile = tileMap[1,1]
+        tile.baseTerrain ="Grassland"
+        tile.addTerrainFeature("Forest")
+
+        val improvement = testGame.createTileImprovement()
+        Assert.assertFalse("Forest doesn't allow building unless allowed",
+            tile.improvementFunctions.canBuildImprovement(improvement, civInfo))
+
+
+        val allowedImprovement = testGame.createTileImprovement()
+        allowedImprovement.terrainsCanBeBuiltOn += "Forest"
+        Assert.assertTrue("Forest should allow building when allowed",
+            tile.improvementFunctions.canBuildImprovement(allowedImprovement, civInfo))
+        tile.setImprovement(allowedImprovement.name)
+        Assert.assertTrue(tile.improvement == allowedImprovement.name)
+        Assert.assertTrue("Forest should not be removed with this improvement", tile.terrainFeatures.contains("Forest"))
+    }
+
+    @Test
+    fun improvementDoesntNeedRemovalCanBuildHere() {
+        val tile = tileMap[1,1]
+        tile.baseTerrain ="Grassland"
+        tile.addTerrainFeature("Forest")
+
+        val improvement = testGame.createTileImprovement("Does not need removal of [Forest]")
+        Assert.assertTrue(tile.improvementFunctions.canBuildImprovement(improvement, civInfo))
+        tile.setImprovement(improvement.name)
+        Assert.assertTrue(tile.improvement == improvement.name)
+        Assert.assertTrue("Forest should not be removed with this improvement", tile.terrainFeatures.contains("Forest"))
+    }
+
+    @Test
+    fun statsDiffFromRemovingForestTakesRemovedLumberMillIntoAccount() {
         val tile = tileMap[1,1]
         tile.baseTerrain = "Grassland"
         tile.addTerrainFeature("Forest")
 
         val lumberMill = testGame.ruleset.tileImprovements["Lumber mill"]!!
-        tile.improvementFunctions.changeImprovement(lumberMill.name)
+        tile.improvementFunctions.setImprovement(lumberMill.name)
         assert(tile.getTileImprovement() == lumberMill)
 
         // 1f 1p from forest, 2p from lumber mill since all techs are researched
